/*
 * Copyright (c) 2021-2025 fausto@disroot.org
 * r4f5fajp2oiuhjqyass3nogu55sivxnc42fnwankiynf6o5jpno6exad.onion/tc.html
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <assert.h>
#include <arpa/inet.h>
#include <ctype.h>
#include <err.h>
#include <errno.h>
#include <getopt.h>
#include <curses.h>
#include <netdb.h>
#include <netinet/in.h>
#include <pthread.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <syslog.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#ifdef linux
#include <bsd/string.h>
#endif
#include "fmt_scaled.h"

#ifdef netbsd
#define GETSTRING(M,S) getnstr(M, S+1); deleteln();
#else
#define GETSTRING(M,S) getnstr(M, S); deleteln();
#endif

#define TCURL "r4f5fajp2oiuhjqyass3nogu55sivxnc42fnwankiynf6o5jpno6exad.onion"
#define VERSION        "1.2"
#define BUILD          "20250528"
#define MAX_CONTACTS   100           /* contacts limit                  */
#define MAX_GROUPS     50            /* groups limit                    */
#define MAX_MEMBERS    5             /* members per group limit         */
#define MAX_SNDFSIZE   3000          /* file size byte limit            */
#define MAX_FSIZE      11000         /* tc.conf file size limit         */
#define N              10            /* nickname lenght and some limits */
#define NFILE          20            /* files loaded by localfilez()    */
#define COMMANDS       41
#define FLAGS          13
#define ALIVE          30            /* socket timeval in seconds       */
#define IN             60            /* input text lenght               */
#define BACKLOG        64            /* pending connections queue hold  */
#define MSG            75            /* multirole string message & more */
#define STR            200           /* multirole string system & more  */
#define RWLENGTH       256           /* fread/fwrite lenght             */
#define SKA            8             /* symmetric-key algorithms        */
#define FMT            32            /* util.h FMT_SCALED_STRSIZE 32    */
#define BSIZE          0x1000        /* to curl the ip address web page */

#define CTRL(C)        ((C) & 037)
#define INFOBOX        info = newwin(17,55,3,12); box(info, 0, 0);
#define REPOSITION(F)  if(fseek(F,0L,SEEK_END) != 0) errx(1,"reposition error");
#define LOCK(M)        if(pthread_mutex_lock(M) != 0) errx(1, "lock fail");
#define UNLOCK(M)      if(pthread_mutex_unlock(M) != 0) errx(1, "unlock fail");
#define BADPCLOSE(P)   if(pclose(P) == -1) superbad();
#define TIMESCAPE \
    time(&t); if(localtime_r(&t,&tm) == NULL) errx(1,"localtime_r(3) failed");

int showfile(char const *file);
int topic(char *m);

struct {

    char name[N];                  /* contact's name in tc.conf       */
    char key[41];                  /* contact's pgp key id            */
    char onion[63];                /* contact's onion addess          */

} c[MAX_CONTACTS];

struct {

    char grpname[N];               /* group name                      */
    char members[MAX_MEMBERS][N];  /* group members                   */

} g[MAX_GROUPS];

struct {

    char keyid[41];                /* pgp key ids                     */
    char descr[70];                /* pgp key description mail        */

} ring[MAX_CONTACTS];

struct {

    char onion[64];                /* saved onion address             */

} restore[N];

struct {

    char auth[STR];                /* .auth files                     */

} aauth[N];

struct {

    char prv[STR];                 /* .auth_private files             */

} aprv[N];

struct tm tm;                      /* time                            */
time_t tstart, t, tkep, tflo;      /* tc uptime and time              */
pthread_t clk, srv, dec;           /* clocker listener send_decoy     */
pthread_mutex_t mtxtct;            /* lock TCT write in ucrypt/gcrypt */
pthread_mutex_t mtxtcr;            /* lock TCR write while send file  */
WINDOW *mainwin,*txt,*pad,*info;   /* all windows                     */
pid_t tpid = 0;                    /* Tor process id                  */
long long bytein, byteout;         /* tc cyphered traffic counters    */
const char *home, *jail, *trm;     /* const because unveil()          */

unsigned int day, hour, min, sec;
int TPORT, TCPORT, burst, nmex, ndec, percent;
short decoy, lazy, logs, voice, sym, away, tor, flood, con, grp, dre, x, y,
      noblank, scrollback, nusr, ngrp, nkey, nrow, who, hpos, row, col, info_f,
      info_b, bars_f, bars_b, text_f, text_b, warn_f, warn_b, strict,
      nauth, nprv, camuflage, symg, symo, snd, theylive, ibox, undead, pager,
      nap;
char *TCS, *TCT, *TCR, *TCN, *tcconf, *tclog, *tclock, *tcbot, *tcbotsh, *tcdir,
     *tcrcvfile, *tcrc, *tchost, *tcauthdir, *tchiddendir, *tcauthclidir,
     *tctordir, *tcver, *tctgz, *pem, *k[5], *o[5], chk[17], chktor[17], K[MSG],
     in[MSG], msg[MSG], way[MSG], zzz[21], nick[N], remo[N], flag[FLAGS],
     PASS[MSG], TPROXY[16], TCDAEMON[16], tport[6], s[MSG][MSG], socks4a[STR],
     callo[63], callk[63], gpgver[N], torver[N], cipher[12];
char tag[]  = "tc - anonymous chat over Tor in PGP";
char *commands[COMMANDS] = {"/algo","/away","/bot","/call","/camuflage",
     "/circuit","/clear","/colors","/date","/decoy","/disconnect","/exit",
     "/flags","/flood","/generate","/groups","/help","/history","/info","/join",
     "/keepalive","/keys","/lazy","/log","/me","/names","/nick","/part","/quit",
     "/restore","/send","/show","/strict","/symmetric","/topic","/uptime",
     "/version", "/voice","/who","/whois",0};
char *flags[FLAGS] = {"[D] decoy","[L] log","[V] text to speach",
     "[S] symmetric", "[A] away", "[T] tor","[F] antiflood","[Z] lazyness",
     "[X] authorize", "[C] camuflage","[G] group","[K] keepalive",0};
char *alg[SKA] = {"AES","AES192","AES256","CAMELLIA128","CAMELLIA192",
     "CAMELLIA256","TWOFISH",0};
char *colors[9] = {"BLACK","RED","GREEN","YELLOW","BLUE","MAGENTA","CYAN",
     "WHITE",0};
void c_info(void) {init_pair(1,info_f,info_b);
     wattron(txt, A_BOLD | COLOR_PAIR(1));}
void c_info_off(void) {wattroff(txt, A_BOLD | COLOR_PAIR(1));}
void c_bars(void) {init_pair(2,bars_f,bars_b); color_set(2,NULL);}
void c_text(void) {init_pair(3,text_f,text_b);
     wattron(txt, A_NORMAL | COLOR_PAIR(3));}
void c_text_off(void) {wattroff(txt, A_NORMAL | COLOR_PAIR(3));}
void c_warn(void) {init_pair(4,warn_f,warn_b);
     wattron(txt, A_BOLD | COLOR_PAIR(4));}
void c_warn_off(void) {wattroff(txt, A_BOLD | COLOR_PAIR(4));}
void c_whit(void) {init_pair(5,7,0); wattron(txt, A_BOLD | COLOR_PAIR(5));}
void c_whit_off(void) {wattroff(txt, A_BOLD | COLOR_PAIR(5));}
void c_prompt(void) {init_pair(6,text_f,text_b); color_set(6,NULL);}
void c_oldtrm(void) {init_pair(7,0,7);
     wattron(mainwin, WA_REVERSE | COLOR_PAIR(7));}
void c_oldtrm_off(void) {wattroff(mainwin, WA_REVERSE | COLOR_PAIR(7));}
void hm(void) { wprintw(txt, "%02d:%02d ", tm.tm_hour, tm.tm_min); }
void i_info(void) {init_pair(1,info_f,info_b);
     wattron(info, A_BOLD | COLOR_PAIR(1));}
void i_info_off(void) {wattroff(info, A_BOLD | COLOR_PAIR(1));}
void i_whit(void) {init_pair(5,7,0); wattron(info, A_BOLD | COLOR_PAIR(5));}
void i_whit_off(void) {wattroff(info, A_BOLD | COLOR_PAIR(5));}

void colorinfo(void) {

    hm(); c_info(); wprintw(txt, "<*> "); c_info_off();
}

void colorwarn(void) {

    hm(); c_warn(); wprintw(txt, "<!> "); c_warn_off();
}

void usage(void) {

    errx(1, "[-n nickname] [-s nickname]");
}

void superbad(void) {

    err(1,NULL); exit(1);
}

unsigned hash(char *str) { /* ripped from K&R */

   unsigned hashval;
   for(hashval = 0; *str != '\0'; str++)
       hashval = *str + 31 * hashval;
   return hashval % 11111;
}

char const *ska(long long val) {

    char *name = (char *) calloc(1, 10);
    if(name == NULL) { free(name); return NULL; }
    if(!fmt_scaled(val, name)) return name;
    else superbad();
    return NULL;
}

void test4oldterm(void) {

    if(!strcmp("vt100", trm) || !strcmp("vt220", trm )) c_oldtrm();
    else c_bars();
}

void waitingfor(void) {

    deleteln(); mvprintw((row-1),0, " =--> [*] wait...");
}

void prompy(char *prompt) {

    int l = strlen(prompt);
    deleteln();
    mvprintw((row-1), 0, " =--> %s", prompt);
    move((row-1), 6+l);
    echo();
}

void help(void) {

    ibox = 1;
    INFOBOX
    int r = 2;
    i_info(); mvwprintw(info,1,23,"commands"); i_info_off();
    for(int i = 0; i < (COMMANDS - 1); i++) {

        if(commands[i] == 0) break;
        mvwprintw(info,r,5,"%s",commands[i]); i++;
        if(commands[i] == 0) break;
        mvwprintw(info,r,23,"%s",commands[i]); i++;
        if(commands[i] == 0) break;
        mvwprintw(info,r,41,"%s",commands[i]); r++;
    }
    wrefresh(info); prompy(""); noecho(); getch(); delwin(info);
    ibox = 0;
}

void flagz(void) {

    ibox = 1;
    short tic[FLAGS];
    tic[0] = decoy;  tic[1] = logs;      tic[2] = voice; tic[3] = sym;
    tic[4] = away;   tic[5] = tor;       tic[6] = flood; tic[7] = lazy;
    tic[8] = strict; tic[9] = camuflage; tic[10]= grp; tic[11] = undead;
    INFOBOX
    int r = 1;
    i_info(); mvwprintw(info,1,23,"flags"); i_info_off();
    for(int i = 0; i < (FLAGS-1); i++) {

        r = r + 2;
        if(tic[i] != 0) i_whit(); mvwprintw(info,r,9,"%s",flags[i]);
        i_whit_off();
        if(i == (FLAGS-2)) break;
        i++;
        if(tic[i] != 0) i_whit(); mvwprintw(info,r,34,"%s",flags[i]);
        i_whit_off();
    }
    wrefresh(info); prompy(""); noecho(); getch(); delwin(info);
    ibox = 0;
}

int logger(char *nn, char *mm) {

    if(strstr(mm, " activated symmetric ") != NULL) return(1);
    if(nn == NULL) {

        FILE *f = fopen(tclog, "a");
        if(f != NULL) {

            fprintf(f, "%02d:%02d %s\n", tm.tm_hour, tm.tm_min, mm);
            fclose(f);
        }
        else return(1);
    }
    else {

        FILE *f = fopen(tclog, "a");
        if(f != NULL) {

            fprintf(f, "%02d:%02d <%s> %s\n",tm.tm_hour,tm.tm_min,nn,mm);
            fclose(f);
        }
        else return(1);
    }
    return(0);
}

char *deblank(char *str) {

    char *out = str, *put = str;
    for(; *str != '\0'; ++str) {

        if(*str != '>' && *str != '<' && *str != ';' && *str != ' '
        && *str != '`' && *str != '&' && *str != '\\' && *str != '"'
        && *str != '|' && *str != '/' && *str != '\'' && *str != '#'
        && *str != '$' && *str != '*' && *str != '[' && *str != ']'
        && *str != '{' && *str != '}' && *str != '@' && *str != '%')

            *put++ = *str;
    }
    *put = '\0';
    return out;
}

char *getonion() {

    char *n = (char *) calloc(1, 63);
    if(n == NULL) { free(n); return NULL; }
    prompy("[*] onion: "); GETSTRING(n, 63-1);
    if(n[0] != '\0' && n[0] != '\n' && n[56] == '.' && n[57] == 'o' &&
       n[58] == 'n' && n[59] == 'i' && n[60] == 'o' && n[61] == 'n') {

       char *d = deblank(n);
       if(strlen(d) != 62) goto ooops;
       else return d;
    }
    else goto ooops;

ooops:
    free(n); colorwarn(); wprintw(txt, "invalid hostname\n"); return NULL;
}

const char *mkfake(int camuf) { /* ripped from ~el8[1] zine ahahah */

    if(camuf == 0) {

    char *name = (char *) calloc(1, 10);
    if(name == NULL) { free(name); return NULL; }
    *(name + 0) = 'A' + (lrand48() % ('Z' - 'A'));
    *(name + 1) = 'a' + (lrand48() % ('z' - 'a'));
    *(name + 2) = 'A' + (lrand48() % ('Z' - 'A'));
    *(name + 3) = 'a' + (lrand48() % ('z' - 'a'));
    *(name + 4) = 'A' + (lrand48() % ('Z' - 'A'));
    *(name + 5) = 'a' + (lrand48() % ('z' - 'a'));
    *(name + 6) = 'A' + (lrand48() % ('Z' - 'A'));
    *(name + 7) = 'a' + (lrand48() % ('z' - 'a'));
    *(name + 8) = 'A' + (lrand48() % ('Z' - 'A'));
    *(name + 9) = 'a' + (lrand48() % ('z' - 'a'));
    *(name + 10) = 0;
    return name;
    }
    if(camuf == 1) {

        int n = 19;
        const char *str[] = { /* push different crap anytime */
                        "....'.WB..E...C..@.@.x.......0.pool.ntp.org",
                        "..........UD....abcdefghijklmnopqrstuvwabcdefghi",
                        "%PDF-1.5.%......779 0 obj.<</Linearized 1/L 5187",
                        "......JFIF.............C........................",
                        "... ftypisom....isomiso2avc1mp41....freeJ...mdat",
                        "GIF89a..>...1...$..H..l...............$.$$.H$.l$",
                        "@charset utf-8;  /* Reset CSS */  body, h1, h2, ",
                        "/*! jQuery v@1.8.0 jquery.com | jquery.org/licen",
                        ".",
                        "..",
                        ".<DOCTYPE html>.",
                        "...F...",
                        "...x..i#ZZ9?.",
                        "font-size: 30px;",
                        "text-align:center; border:none; border-radius:5px;",
                        "*//*! normalize.css v3.0.3 | MIT License | ",
                        ",up.each({scrollLeft:pageXOffset,scrollTop:page11",
                        ".zcss(a,boxSizing)===border-box;..",
                        0
                       };
        int vs = lrand48() % (n-1);
        return str[vs];
    }
    if(camuf == 2) {

        int n = 19;
        const char *str[] = {
        "Lynx/2.9.2 libwww-FM/2.14 SSL-MM/1.4.",
        "curl/7.74.0",
        "curl/8.12.0",
        "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 18_4_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/18.0 Mobile/15E148 Safari/605.1.15",
        "User-Agent: Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/135.0.0.0 Mobile Safari/537.36",
        "User-Agent: Apache-HttpClient/4.5.13 (Java/1.8.0_442)",
        "User-Agent: Opera/9.80 (Windows NT 6.1) Presto/2.12.388 Version/12.17",
        "User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 18_3_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/18.0 Mobile/15E148 Safari/605.1.15",
        "User-Agent: Mozilla/5.0 (iPad; CPU iPhone OS 17_7_5 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Version/17.7.5 Safari/605.1.15 AlohaBrowser/7.0.0",
        "User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:128.0) Gecko/20100101 Firefox/128.0",
        "User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/135.0.0.0 Safari/537.36",
        "User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36",
        "User-Agent: Mozilla/5.0 (Windows NT 10.0; rv:102.0) Gecko/20100101 Firefox/102.0",
        "User-Agent: Mozilla/5.0 (Linux; Android 9; SM-A107F) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.136 Mobile Safari/537.36",
        "User-Agent: Mozilla/5.0 (Android 10; Mobile; rv:109.0) Gecko/115.0 Firefox/115.0",
        "User-Agent: Mozilla/5.0 (Android 14; Mobile; rv:128.0) Gecko/128.0 Firefox/128.0",
        "User-Agent: Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36",
        "User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
        "User-Agent: Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.172 Safari/537.22",
        0
        };
        int vs = lrand48() % (n-1);
        return str[vs];
    }
    return NULL;
}

int shred(char *file) {

    int i = 0, j = 0 ;

zoot:

    if(!j) j = 1;
    else   j = 0;

    FILE *f = fopen(file, "w");
    if(f != NULL) {

        char buf[MAX_SNDFSIZE] = {"\0"};
        memset(buf, j, sizeof(buf)); fprintf(f, "%s", buf);
        fflush(f); fclose(f);
    }
    if(i == 2) { if(unlink(file) != 0) superbad(); return(0); }
    else { i++; goto zoot; }
    return(0);
}

void nocom(void) {

    if(decoy) { if(pthread_cancel(dec) != 0) superbad(); else decoy = 0; }
    unlink(TCS); shred(TCT); unlink(TCR); shred(TCN);
    con = 0; grp = 0; who = 0; symg = 0; symo = 0; undead = 0;
    memset(remo, 0, sizeof(remo)); memset(k, 0, sizeof(k));
    memset(o, 0, sizeof(o)); memset(zzz, 0, sizeof(zzz));
    colorwarn(); wprintw(txt, "not connected\n");
}

void hexdumpo(unsigned char const *data) {

    for(int i = 0; i < 100; i += 16) {

        hm(); wprintw(txt, "%04x: ", i);
        for (int j = 0; j < 16; j++) {

            wprintw(txt,"%02x ", data[i+j] & 0xff);
        }
        wprintw(txt, "|");
        for (int z = 0; z < 16; z++) {

            int hex = data[i+z] & 0xff;
            if(!isprint(hex)) hex = '.'; wprintw(txt, "%c", hex);
        }
        wprintw(txt, "|\n");
    }
}

void elapse(time_t start) { /* almost ripped from top(1)'s display.c */

    time_t elapse, stop;
    stop = time(&t);
    elapse = difftime(stop,start);
    day = elapse / (3600 * 24);
    elapse %= (3600 * 24);
    hour = elapse / 3600;
    elapse %= 3600;
    min = elapse / 60;
    sec = elapse % 60;
}

void draw(void) {

    char bar[400] = "";
    for(int b = 0; b < col; b++) bar[b] = ' ';
    test4oldterm();
    mvprintw(0, 0, "%s", bar);
    mvprintw(0, 1, "%s", tag);
    mvprintw((row-2), 0, "%s", bar);
    (decoy) ? memset(&flag[0], 'D', 1) : memset(&flag[0], ' ', 1);
    (logs) ? memset(&flag[1], 'L', 1) : memset(&flag[1], ' ', 1);
    (voice) ? memset(&flag[2], 'V', 1) : memset(&flag[2], ' ', 1);
    (sym) ? memset(&flag[3], 'S', 1) : memset(&flag[3], ' ', 1);
    (away) ? memset(&flag[4], 'A', 1) : memset(&flag[4], ' ', 1);
    (tor) ? memset(&flag[5], 'T', 1) : memset(&flag[5], ' ', 1);
    (flood) ? memset(&flag[6], 'F', 1) : memset(&flag[6], ' ', 1);
    (lazy) ? memset(&flag[7], 'Z', 1) : memset(&flag[7], ' ', 1);
    (strict) ? memset(&flag[8], 'X', 1) : memset(&flag[8], ' ', 1);
    (camuflage) ? memset(&flag[9], 'C', 1) : memset(&flag[9], ' ', 1);
    (grp) ? memset(&flag[10], 'G', 1) : memset(&flag[10], ' ', 1);
    (undead != 0) ? memset(&flag[11], 'K', 1) : memset(&flag[11], ' ', 1);
    mvprintw((row-2),0, " [%02d:%02d] [%s] remote is [%s] [%s] ",
    tm.tm_hour, tm.tm_min, nick, remo, flag);
    test4oldterm();
    if(nap) {

        memset(zzz,0,sizeof(zzz)); mvprintw((row-2),col-10,"... zZZzz");
    }
    if(scrollback || pager) {

        memset(zzz,0,sizeof(zzz)); mvprintw((row-2),col-7,"-more-");
    }
    if(flood && (nmex >= burst)) {

        memset(zzz,0,sizeof(zzz)); mvprintw((row-2),col-8,"-flood-");
    }
    if(theylive) {

        memset(zzz,0,sizeof(zzz)); mvprintw((row-2),col-6,"-tor-");
    }
    if(snd) {

        int l = strlen(zzz);
        mvprintw((row-2),col-15, "               ");
        mvprintw((row-2),col-6-l,"%s", zzz);       /* amount transferred  */
        mvprintw((row-2),col-6,"(%d%%)", percent); /* percent transferred */
        mvprintw((row-2),col-1, " ");
        refresh();
    }
    c_oldtrm_off();
    c_prompt();
    mvprintw((row-1), 0, " =--> ");
    if(dre) { move(y, x); dre = 0; }
    else if(ibox && theylive) { redrawwin(info); wrefresh(info); }
    else if(!scrollback && ibox) {

        wrefresh(txt); wrefresh(mainwin); redrawwin(info); wrefresh(info);
    }
    if(!scrollback) { wrefresh(txt); wrefresh(mainwin); }
}

void bannerz(void) {

    ibox = 1;
    info = newwin(14,74,3,3); box(info, 0, 0);
    i_info(); mvwprintw(info,1,33,"got tc?"); i_info_off();
    mvwprintw(info,3,26,"enjoy the update from");
    mvwprintw(info,5,2,"%s/tc.html", TCURL);
    mvwprintw(info,7,13,"donate because than I can get some money to buy");
    mvwprintw(info,8,26,"lunch and skateboards");
    mvwprintw(info,10,13,"BTC: bc1qwzj9tk9vym63ddfh9v8xt0guxw47mxm8kwjpkt");
    wrefresh(info); sleep(3); ibox = 0; getch(); redrawwin(info);
}

int torpid(void) {

    FILE *p = NULL;
    if((p = popen("ps xa | grep \"tor \" | head -n1 | \
    grep -v \"grep \" | awk '{print $1}'", "r")) == NULL) {

        tpid = 0; tor = 0; return(1);
    }
    char tp[N] = {"\0"}; fgets(tp, N, p);
    tpid = strtoul(tp, NULL, N); BADPCLOSE(p)
    tor = (tpid != 0) ? 1 : 0;
    return(0);
}

int killtor(void) {

    FILE *p = NULL;
    if((p = popen("ps aux | grep \"tor \" | head -n1 | \
    awk '{print $1}'", "r")) == NULL) return(1);
    char u[20] = {"\0"};
    fgets(u, 20, p); u[strcspn(u, "\n")] = '\0';
    BADPCLOSE(p)
    char me[9] = {'\0'}; /* LOGIN_NAME_MAX limits.h/syslimits.h */
    if(getlogin_r(me, sizeof(me)) != 0) superbad();
    if(strcmp(u,me) != 0) {

        colorwarn();
        wprintw(txt, "can't kill Tor (process owner: %s, me: %s)\n",u,me);
        return(1);
    }
    else {

        if(system("pkill -x tor") == -1) {

            colorwarn(); wprintw(txt, "pkill: %s\n", strerror(errno));
        }
        return(0);
    }
}

char *hidden(void) {

    if(!access(tchost, F_OK)) {

        FILE *f = fopen(tchost, "r");
        if(f == NULL) return NULL;
        else {

            char *hostname = (char *) calloc(1, 64);
            if(hostname == NULL) { free(hostname); fclose(f); return NULL; }
            else {

                fgets(hostname,63,f); fclose(f); return hostname;
            }
        }
    }
    else return NULL;
}

int fireuptor(void) {

    sleep(1);
    char buf[STR] = {"\0"};
    strlcat(buf, "tor -f ", sizeof(buf));
    strlcat(buf, tcrc, sizeof(buf));
    strlcat(buf, " >/dev/null 2>&1", sizeof(buf));
    FILE *p = NULL;
    if((p = popen(buf, "r")) == NULL) return(1);
    BADPCLOSE(p)
    sleep(5);
    return(0);
}

void printkauth(void) {

    hm(); c_info(); wprintw(txt,"<*> private keys from $HOME/.tc/tor/auth");
    c_info_off();
    for(int i = 0; i < nauth; i++) {

        if(i%2 == 0) {

            wprintw(txt,"\n"); hm();
        }
        wprintw(txt,"  %d.[ ", i);
        if(strstr(aauth[i].auth, "@") != NULL) c_whit();
        if(strstr(aauth[i].auth, ".auth_private") != NULL &&
           !strstr(aauth[i].auth, "@")) c_warn();
        wprintw(txt,"%-28s",aauth[i].auth); c_whit_off(); c_warn_off();
        wprintw(txt," ]");
    }
    wprintw(txt,"\n"); hm(); wprintw(txt,"(total count: %d)\n",nauth);
}

void printkprv(void) {

    hm(); c_info(); wprintw(txt,
    "<*> public keys from $HOME/.tc/tor/hidden_service/authorized_clients");
    c_info_off();
    for(int i = 0; i < nprv; i++) {

        if(i%2 == 0) {

            wprintw(txt,"\n"); hm();
        }
        wprintw(txt,"  %d.[ ", i);
        if(strstr(aprv[i].prv, "@") != NULL) c_whit();
        if(strstr(aprv[i].prv, ".auth") != NULL &&
           !strstr(aprv[i].prv, ".auth_private") &&
           !strstr(aprv[i].prv, "@")) c_warn();
        wprintw(txt,"%-28s",aprv[i].prv); c_whit_off(); c_warn_off();
        wprintw(txt," ]");
    }
    wprintw(txt,"\n"); hm(); wprintw(txt,"(total count: %d)\n",nprv);
}

int checkfsize(FILE *fp, char *f, long long z, int cli) {

    if(fp == NULL) return(1);
    else {

        REPOSITION(fp)
        long long siz = ftell(fp); fclose(fp);
        char skaled[FMT];
        strlcpy(skaled, ska(siz), sizeof(skaled));
        if((siz > z) && !cli) {

            colorinfo();
            wprintw(txt, "%s is a big one (%s), take some times\n",f,skaled);
        }
        if(((siz > z) || (siz <= 0)) && cli) {

            errx(1, "%s empty or greater than %lld. (%s)",f,z,skaled);
        }
    }
    return(0);
}

void sketchyconf(FILE *f) {

    if(feof(f)) {

        endwin();
        errx(1, "sketchy tc.conf incomplete or corrupted");
    }
}

int loadconfig(void) {

    FILE *f = fopen(tcconf, "r"); if(f == NULL) superbad();

    size_t len = 0;
    ssize_t linelen;
    char *b = NULL;
    char *line = NULL;

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else { b[strcspn(b,"\n")]='\0'; }
    if(strcmp(b,"# passphrase"))
        errx(1,"irregular tc.conf (# passphrase)");
    sketchyconf(f);

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else b[strcspn(b,"\n")]='\0';
    strlcpy(PASS, b, sizeof(PASS));
    sketchyconf(f);

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else b[strcspn(b,"\n")]='\0';
    if(strcmp(b,"# Tor proxy address and port"))
        errx(1,"irregular tc.conf (# Tor proxy address and port)");
    sketchyconf(f);

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else b[strcspn(b,"\n")]='\0';
    strlcpy(TPROXY, strsep(&b, ":"), sizeof(TPROXY));
    strlcpy(tport, strsep(&b, ":"), sizeof(tport));
    TPORT = atoi(tport);
    sketchyconf(f);

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else b[strcspn(b,"\n")]='\0';
    if(strcmp(b,"# tc address and port"))
        errx(1,"irregular tc.conf (# tc address and port)");
    sketchyconf(f);

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else b[strcspn(b,"\n")]='\0';
    strlcpy(TCDAEMON, strsep(&b, ":"), sizeof(TCDAEMON));
    TCPORT = atoi(strsep(&b, ":"));
    sketchyconf(f);

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else b[strcspn(b,"\n")]='\0';
    if(strcmp(b,"# colors"))
        errx(1,"irregular tc.conf (# colors)");
    sketchyconf(f);

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else b[strcspn(b,"\n")]='\0';
    line = b;
    info_f = atoi(strsep(&line, ":"));
    info_b = atoi(strsep(&line, ":"));
    sketchyconf(f);

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else b[strcspn(b,"\n")]='\0';
    line = b;
    bars_f = atoi(strsep(&line, ":"));
    bars_b = atoi(strsep(&line, ":"));
    sketchyconf(f);

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else b[strcspn(b,"\n")]='\0';
    line = b;
    text_f = atoi(strsep(&line, ":"));
    text_b = atoi(strsep(&line, ":"));
    sketchyconf(f);

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else b[strcspn(b,"\n")]='\0';
    line = b;
    warn_f = atoi(strsep(&line, ":"));
    warn_b = atoi(strsep(&line, ":"));
    sketchyconf(f);

    linelen = getline(&b, &len, f);
    if(linelen == -1 || linelen == 0) superbad();
    else b[strcspn(b,"\n")]='\0';
    if(strcmp(b,"# contacts"))
        errx(1,"irregular tc.conf (# contacts)");
    sketchyconf(f);

    int i = 0;
    while ((linelen = getline(&b, &len, f)) || !strcmp(b,"#")) {

        if(linelen == -1 || linelen == 0) superbad();
        sketchyconf(f);
        line = b;
        if(line == NULL) break;
        if(!strcmp(line, "\n")) break;
        if(strstr(line, ":") == NULL) break;
        strlcpy(c[i].name, strsep(&line, ":"), sizeof(c[i].name));
        strlcpy(c[i].key, strsep(&line, ":"), sizeof(c[i].key));
        strlcpy(c[i].onion, strsep(&line, ":"), sizeof(c[i].onion));
        i++;
        nusr++;
        if(i >= MAX_CONTACTS) break;
    }
    if(linelen == -1 || linelen == 0) superbad();
    b[strcspn(b, "\n")] = '\0';
    if(strcmp(b,"# groups"))
        errx(1,"irregular tc.conf (# groups)");
    sketchyconf(f);

    i = 0;
    while ((linelen = getline(&b, &len, f)) || !strcmp(b,"#")) {

        if(linelen == -1 || linelen == 0) superbad();
        sketchyconf(f);
        line = b;
        if(line == NULL) break;
        if(!strcmp(line, "\n")) break;
        if(strstr(line, "# eof") != NULL) break;
        strlcpy(g[i].grpname, strsep(&line, ":"),
        sizeof(g[i].grpname));

        for(int j = 0; j < MAX_MEMBERS; j++) {

            if(line == NULL) break;
            if(strstr(line, ",") == NULL) break;
            strlcpy(g[i].members[j], strsep(&line, ","),
            sizeof(g[i].members[j]));
        }
        i++;
        ngrp++;
        if(i >= MAX_GROUPS) break;
    }
    if(linelen == -1 || linelen == 0) superbad();
    free(b);
    if(ferror(f)) superbad();
    fclose(f); return(0);
}

int gettorver(void) {

    FILE *f = NULL;
    if((f = popen("tor --version | head -n1 | cut -c 13-20","r")) == NULL)
    return(1);
    fgets(torver, N, f);
    torver[strcspn(torver, "\n")] = '\0';
    BADPCLOSE(f)
    return(0);
}

int getgnupgver(void) {

    FILE *f = NULL;
    if((f = popen("gpg --version | head -n1 | cut -c 13-17","r")) == NULL)
    return(1);
    fgets(gpgver, N, f);
    gpgver[strcspn(gpgver, "\n")] = '\0';
    BADPCLOSE(f)
    return(0);
}

int loadkeysid(void) {

    memset(&ring, 0, sizeof(ring)); /* clear struct */
    /* since v2.4.1 gnupg signature list show [self-signature] not uid */
    int finger = 0;
    FILE *p = NULL;
    if((p = popen("gpg --no-option --list-sigs | \
    grep \"sig 3\" | cut -c 14-29", "r")) == NULL) return(1);
    while(1) {

        if(fgets(ring[nkey].descr, 60, p) == NULL) break;
        if(ring[nkey].descr[0] == '\0') break;
        strlcpy(ring[nkey].keyid, ring[nkey].descr, 17);
        if(ring[nkey].keyid[0] != '\n') {

            if(ring[nkey].keyid[0] == '\0') break;
            ring[nkey].keyid[strcspn(ring[nkey].keyid, "\n")] = '\0';
            nkey++;
        }
        else ring[nkey].keyid[strcspn(ring[nkey].keyid, "\n")] = '\0';

        if(nkey >= MAX_CONTACTS) break;
    }
    BADPCLOSE(p)
    p = NULL;
    if((p = popen("gpg --no-option --list-sigs | \
    grep \"uid\" | cut -c 25-80", "r")) == NULL) return(1);
    int nk = 0;
    char buf[65] = {"\0"};
    while(1) {

        if(fgets(ring[nk].descr, 65, p) == NULL) break;
        if(ring[nk].descr[0] == '\0') break;
        strlcpy(buf, ring[nk].keyid, sizeof(buf));
        strlcat(buf, ring[nk].descr, sizeof(buf));
        strlcpy(ring[nk].descr, buf, 65);
        if(ring[nk].descr[0] != '\n') {

            if(ring[nk].descr[0] == '\0') break;
            ring[nk].descr[strcspn(ring[nk].descr, "\n")] = '\0';
            nk++;
        }
        else ring[nk].descr[strcspn(ring[nk].descr, "\n")] = '\0';

        if(nk >= MAX_CONTACTS) break;
    }
    BADPCLOSE(p)
    p = NULL; /* ed25519 fingerprint */
    if((p = popen("gpg --no-option --list-sigs |\
    grep -A1 \"pub   ed25519\" | grep -v \"pub   ed25519\" |\
    grep -v -e \"pub   ed25519\" -e \"--\" |\
    cut -c 7-46" , "r")) == NULL) return(1);
    while(1) {

        if(fgets(ring[nkey].descr, 60, p) == NULL) break;
        if(ring[nkey].descr[0] == '\0') break;
        strlcpy(ring[nkey].keyid, ring[nkey].descr, 41);
        if(ring[nkey].keyid[0] != '\n') {

            if(ring[nkey].keyid[0] == '\0') break;
            ring[nkey].keyid[strcspn(ring[nkey].keyid, "\n")] = '\0';
            nkey++;
            finger++;
        }
        else ring[nkey].keyid[strcspn(ring[nkey].keyid, "\n")] = '\0';

        if(nkey >= MAX_CONTACTS) break;
    }
    BADPCLOSE(p)
    p = NULL;  /* ed25519 description */
    if((p = popen("gpg --no-option --list-sigs | \
    grep -A3 \"ed25519\" | grep -v -e \"sig\" -e \"pub\" -e \"sub\" -e \"--\"|\
    grep \"uid\" | cut -c 25-90", "r")) == NULL) return(1);
    nk = (nkey - finger);
    buf[0] = '\0';
    while(1) {

        if(fgets(ring[nk].descr, 64, p) == NULL) break;
        if(ring[nk].descr[0] == '\0') break;
        strlcpy(buf, ring[nk].keyid, sizeof(buf));
        strlcat(buf, ring[nk].descr, sizeof(buf));
        strlcpy(ring[nk].descr, buf, 75);
        if(ring[nk].descr[0] != '\n') {

            if(ring[nk].descr[0] == '\0') break;
            ring[nk].descr[strcspn(ring[nk].descr, "\n")] = '\0';
            nk++;
        }
        else ring[nk].descr[strcspn(ring[nk].descr, "\n")] = '\0';

        if(nk >= MAX_CONTACTS) break;
    }
    BADPCLOSE(p)
    if(ring[0].keyid[0] == 0) return(1);
    return(0);
}

int loadauthkeys(void) {

    memset(&aauth, 0, sizeof(aauth)); /* clear struct */
    nauth = 0;
    char buf[STR] = {"\0"};
    char tmp[29] = {"\0"};
    strlcat(buf, "ls -1 ", sizeof(buf));
    strlcat(buf, tcauthdir, sizeof(buf));
    FILE *p = NULL;
    if((p = popen(buf, "r")) == NULL) return(1);
    while(1) { /* load .auth files */

        tmp[0] = '\0';
        if(fgets(tmp,29,p) == NULL) break;
        if(tmp[0] == '\0') break;
        int l = strlen(tmp);
        if( l == 0 || l > 28 ) break;
        strlcpy(aauth[nauth].auth,tmp,sizeof(aauth[nauth].auth));
        aauth[nauth].auth[strcspn(aauth[nauth].auth,"\n")] = '\0';
        nauth++;
        if(nauth >= N) break; /* limit key/files */
    }
    BADPCLOSE(p)
    memset(&aprv, 0, sizeof(aprv)); /* clear struct */
    nprv = 0; buf[0] = '\0'; tmp[0] = '\0';
    strlcat(buf, "ls -1 ", sizeof(buf));
    strlcat(buf, tcauthclidir, sizeof(buf));
    p = NULL;
    if((p = popen(buf, "r")) == NULL) return(1);
    while(1) { /* load .auth_private files */

        tmp[0] = '\0';
        if(fgets(tmp,29,p) == NULL) break;
        if(tmp[0] == '\0') break;
        int l = strlen(tmp);
        if( l == 0 || l > 28 ) break;
        strlcpy(aprv[nprv].prv,tmp,sizeof(aprv[nprv].prv));
        aprv[nprv].prv[strcspn(aprv[nprv].prv,"\n")] = '\0';
        nprv++;
        if(nprv >= N) break; /* limit key/files */
    }
    BADPCLOSE(p)
    if(aauth[0].auth[0] == 0) return(1);
    if(aprv[0].prv[0] == 0) return(1);
    return(0);
}

void onoff(int i, int onoff, int a) {

    char buf[STR] = {"\0"}; char f[STR] = {'\0'};
    if(!a) { /* client side auth for remote access */

        strlcat(buf, tcauthdir, sizeof(buf));
        strlcat(buf, "/", sizeof(buf));
        strlcat(buf, aauth[i].auth, sizeof(buf));
    }
    if(a) { /* server side auth for local access */

        strlcat(buf, tcauthclidir, sizeof(buf));
        strlcat(buf, "/", sizeof(buf));
        strlcat(buf, aprv[i].prv, sizeof(buf));
    }
    strlcat(f, buf, sizeof(f));
    const unsigned int l = strlen(f);
    if(onoff && l > 0 && f[l-1] == '@') { /* enable files */

            f[l-1] = '\0'; rename(buf, f); strict = 1;
    }
    if(!onoff && l > 0 && f[l-1] != '@') { /* disable files */

            strlcat(f, "@", sizeof(f)); rename(buf, f); strict = 0;
    }
}

void mutual(int m) {

    loadauthkeys();
    if(m) { /* enable all keys */

        for(int i = 0; i < nauth; i++)
        if(strstr(aauth[i].auth,".auth_private@") != NULL) onoff(i,1,0);
        for(int i = 0; i < nprv; i++)
        if(strstr(aprv[i].prv,".auth@") != NULL) onoff(i,1,1);
    }
    if(!m) { /* disable all keys */

        for(int i = 0; i < nauth; i++)
        if(strstr(aauth[i].auth,".auth_private") != NULL) onoff(i,0,0);
        for(int i = 0; i < nprv; i++)
        if(strstr(aprv[i].prv,".auth") != NULL) onoff(i,0,1);
    }
}

void selectk(int kk) {

    loadauthkeys(); (kk == 0) ? printkauth() : printkprv();
    int j = 0, n = 0;
    do {

        draw(); mvprintw((row-1),0, " =--> [*] selected {%d} or (d)one: ", n);
        move(row-1, 34);
        echo();
        GETSTRING(msg, 1);
        if(msg[0] == '\0' || msg[0] == 'd') continue;
        int w = (atoi(msg));
        if(kk == 0 && w >= 0 && w < N) {

            if(strstr(aauth[w].auth,".auth_private@") != NULL) onoff(w,1,0);
            else if(strstr(aauth[w].auth,".auth_private") != NULL) onoff(w,0,0);
        }
        if(kk == 1 && w >= 0 && w < N) {

            if(strstr(aprv[w].prv,".auth@") != NULL) onoff(w,1,1);
            else if(strstr(aprv[w].prv,".auth") != NULL) onoff(w,0,1);
        }
        j++; n++;

    } while(j < N && (strcmp(msg, "d") != 0) && n < nauth && n < nprv);
}

void nokey(void) {

    colorwarn(); wprintw(txt,"no keys found\n");
}

void cleanup(int sig) {

    bannerz();
    unlink(TCS); shred(TCT); unlink(TCR); shred(TCN); unlink(tclock);
    FILE *f = fopen(tclog, "r");
    if(f != NULL) {

        fclose(f);
        prompy("[*] tc.log not empty. shred it? [y/n]: "); GETSTRING(msg, 1);
        if(msg[0] == 'y') shred(tclog); fprintf(stderr, "\n");
    }
    syslog(LOG_INFO, "can become addictive, use with moderation");
    endwin(); exit(0);
}

void speak(void) {

    char buf[STR] = {"\0"};
    strlcat(buf, "espeak -f ", sizeof(buf));
    strlcat(buf, home, sizeof(buf));
    strlcat(buf, "/.tc/.tcn -a 150 -g 3 -p 30 -s 150 >/dev/null 2>&1",
    sizeof(buf));
    if(system(buf) == -1) {

        colorwarn(); wprintw(txt, "espeak: %s\n", strerror(errno));
    }
}

void addscrollback(char *rmsg) {

    char p[STR] = {"\0"};
    if(nrow == STR) { werase(pad); nrow = 0; }
    if(rmsg == NULL) {

        strftime(p, 7, "%H:%M ", &tm);
        strlcat(p, "<", sizeof(p));
        strlcat(p, nick, sizeof(p));
        strlcat(p, "> ", sizeof(p));
        strlcat(p, msg, sizeof(p));
        strlcat(p, "\n", sizeof(p));
        waddstr(pad, p);
    }
    else {

        strlcat(p, "\n", sizeof(p));
        strftime(p, 7, "%H:%M ", &tm);
        waddstr(pad, p);
        waddstr(pad, rmsg);
        waddstr(pad, "\n");
    }
    nrow++;
}

void drawremote(char *rmsg) {

    /* remote msg include nick; addscrollback() & logger() verify NULL nick */
    if(strstr(rmsg, "<notice> ") != NULL) {

        char buf[MSG] = {"\0"};
        strlcpy(buf, rmsg+13, 70);
        hm(); c_whit(); wprintw(txt, "*notice* "); c_whit_off();
        c_warn(); wprintw(txt, "%s\n", buf); c_warn_off();
        addscrollback(rmsg);
    }
    else if(strstr(rmsg, "<*> ") != NULL) {

        char buf[MSG] = {"\0"};
        strlcpy(buf, rmsg+4, 70);
        colorinfo();
        wprintw(txt, "%s\n", buf);
        addscrollback(rmsg);
    }
    else {

        c_text(); hm(); wprintw(txt, "%s\n", rmsg); c_text_off();
        addscrollback(rmsg);
    }
    if(strstr(rmsg," changed the topic to: ") != NULL) {

        char *m; m = memchr(rmsg, ':', strlen(rmsg)); topic(m+2);
    }
    if(logs) logger(NULL, rmsg);
    dre = 1;
    if(!scrollback || !pager) draw();
    if(voice) speak();
}

int get_page(int s, const char * host, int sox) {

    int status;
    char mex[STR*3] = {'\0'};
    strlcat(mex, "GET / HTTP/1.0\r\n", sizeof(mex));
    strlcat(mex, "Host: ", sizeof(mex));
    strlcat(mex, host, sizeof(mex));
    strlcat(mex, "\r\n", sizeof(mex));
    strlcat(mex, mkfake(2), sizeof(mex));
    strlcat(mex, "\r\n\r\n", sizeof(mex));
    status = send(s, mex, strlen(mex), 0);
    if(status == -1) { close(s); return(1); }
    while (1) {

        int bytes;
        char buf[BSIZE+10];
        bytes = recvfrom(s, buf, BSIZE, 0, 0, 0);
        if(bytes == 0) break;
        if(bytes == -1) break;
        buf[bytes] = '\0';
        int l = strlen(buf);
        char temp[17] = {'\0'};
        char *b = NULL;
        strlcpy(temp, buf+l-31, sizeof(temp));
        temp[strcspn(temp,"<")] = '\0';
        b = temp;
        strsep(&b, " ");
        if(b == NULL) { close(s); return(1); }
        if(sox) {

            memset(chktor, 0, sizeof(chktor));
            strlcpy(chktor, strsep(&b, " "), sizeof(chktor));
            if(chktor[0] != '\0') chktor[strcspn(chktor, "\n")] = '\0';
            else { close(s); return(1); }
        }
        else {

            memset(chk, 0, sizeof(chk));
            strlcpy(chk, strsep(&b, " "), sizeof(chk));
            if(chk[0] != '\0') chk[strcspn(chk, "\n")] = '\0';
            else { close(s); return(1); }
        }
    }
    close(s);
    return(0);
}

void sender(char *onion, char *n, int curloz) {

    int sockfd;
    struct sockaddr_in relay;
    memset(&relay, 0, sizeof(relay)); /* clear struct */
    if((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) exit(1);
    relay.sin_family = AF_INET;
    relay.sin_port = htons(TPORT);
    inet_pton(AF_INET, TPROXY, &relay.sin_addr);
    memset(&(relay.sin_zero), 0, sizeof(relay.sin_zero));
    struct timeval tv;
    tv.tv_sec = ALIVE;
    tv.tv_usec = 0;
    setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
    if(connect(sockfd, (struct sockaddr *)&relay,
    sizeof(struct sockaddr)) == -1) {

        colorwarn();
        wprintw(txt, "Tor proxy unrechable use '/circuit' to verify\n");
        shutdown(sockfd, SHUT_RDWR); close(sockfd); nocom(); return;
    }
    if(lazy) { close(sockfd); return; }
    /*
     *  https://www.openssh.com/txt/socks4.protocol
     *  https://www.openssh.com/txt/socks4a.protocol
     *  ver:4;cd:1;dest port:2323;0.0.0.x;x=1;user:NULL+onion
     *  to check ip address via tor is
     *  ver:4;cd:1;dest port:80;0.0.0.x;x=1;user:NULL+checkip.dyndns.org
     *
     */
    if(curloz) {

        unsigned char biruf[BSIZE+10];
        /*                                port 80                          */
        /*                      ver  cd   00   50   0    0    0    1   user*/
        unsigned char head[9]={0x04,0x01,0x00,0x50,0x00,0x00,0x00,0x01,0x00};
        memcpy(biruf, head, 9);
        memcpy(biruf+9, onion, 18);
        memset(biruf+27, 0, 1);
        send (sockfd, biruf, 28, 0);
        recv(sockfd, biruf, 16, 0);     /* need recieve: 00 5a 00 00 00 */
        if(biruf[0] == 0 && biruf[1] == 90) get_page(sockfd, onion, 1);
        close(sockfd); return;
    }
    unsigned char buf[RWLENGTH];
    if(camuflage) {

        /*                      ver  cd   23   23   0    0    0    1  */
        unsigned char head[8]={0x04,0x01,0x09,0x13,0x00,0x00,0x00,0x01};
        unsigned char zero[1]={0x00};
        char fakie[50] = {"\0"};
        strlcpy(fakie, mkfake(1), sizeof(fakie));
        int lenz = strlen(fakie);
        memcpy(buf, head, 8);
        memcpy(buf+8, fakie, lenz); /* socks4 username filed */
        memcpy(buf+8+lenz, zero, 1);
        memcpy(buf+8+lenz+1, onion, 63);
        send(sockfd, buf, (8+lenz+1+63), 0);
    }
    if(!camuflage) {

        /*                      ver  cd   23   23   0    0    0    1   user*/
        unsigned char head[9]={0x04,0x01,0x09,0x13,0x00,0x00,0x00,0x01,0x00};
        memcpy(buf, head, 9);
        memcpy(buf+9, onion, 63);
        send(sockfd, buf, 72, 0); /* 63+9 = head + onion v3 addr */
    }
    recv(sockfd, buf, 16, 0);     /* need recieve: 00 5a 00 00 00 */
    if(buf[0] || buf[1] != 90) {  /* 5a == 90 */

        nap = 1; draw();
        if(n != NULL && !strcmp("decoy",n)) {

            shutdown(sockfd,SHUT_RDWR); close(sockfd); sleep(1); nap= 0; return;
        }
        colorwarn(); wprintw(txt, "can't send!\n");
        shutdown(sockfd,SHUT_RDWR); close(sockfd); sleep(1); nap = 0; return;
    }
    else {

        FILE *fs = fopen(TCS, "r");
        if(fs == NULL) errx(1, "NULL .tcs or encryption subkey not found\n");
        REPOSITION(fs)
        long long flen = ftell(fs);
        if(fseek(fs, 0L, SEEK_SET) != 0) errx(1, "reposition error");
        memset(buf, 0, sizeof(buf));
        int b = 0;
        char skaled[FMT];
        time_t ssnd = time(&t);
        if(snd) {

            strlcpy(skaled, ska(flen), sizeof(skaled));
            colorinfo(); wprintw(txt, "sending %s (%s)\n", msg, skaled);
        }
        while(1) {

            int block = fread(buf, 1, RWLENGTH, fs);
            if(block) {

                if(feof(fs) != 0 || ferror(fs) != 0) {

                    fseek(fs, 0, SEEK_END); clearerr(fs);
                }
            }
            if(block == 0) break;
            if(send(sockfd, buf, block, 0) < 0) break;
            memset(buf, 0, sizeof(buf));
            if(snd && b++ % 1000 == 0) {

                int transferred = b * RWLENGTH;
                char sent[FMT];
                strlcpy(sent, ska(transferred), sizeof(sent));
                memset(zzz, 0, sizeof(zzz));
                strlcat(zzz, sent, sizeof(zzz));
                if (transferred == 0 || transferred == flen) percent = 100;
                else percent = ((float)transferred / flen) * 100;
            }
        }
        close(sockfd);
        if(snd) {

            elapse(ssnd); colorinfo();
            wprintw(txt, "sent %s in %02u:%02u:%02u\n", skaled, hour, min, sec);
            memset(zzz, 0, sizeof(zzz)); percent = 0;
        }
        byteout =  byteout + flen;
        fclose(fs);
    }
    close(sockfd);
    return;
}

bool isbin(const void *data, size_t len) {

    return memchr(data, '\0', len) != NULL;
}

int checkfile(void) {

    if(access(msg, F_OK) != 0) {

        colorwarn(); wprintw(txt, "%s not exist\n", msg); return(1);
    }
    struct stat st;
    if(!stat(msg, &st)) {

        if(st.st_mode & S_IFDIR) {

            colorwarn(); wprintw(txt, "%s it's a directory\n", msg); return(1);
        }
        if(!st.st_size && S_ISREG(st.st_mode)) {

            colorwarn(); wprintw(txt, "%s it's empty\n", msg); return(1);
        }
        if(st.st_uid != getuid()) {

            colorwarn();
            wprintw(txt, "%s owner uid: %d myself: %d won't open\n",
            msg, st.st_uid, getuid());
            return(1);
        }
        if((st.st_mode & (S_IRUSR|S_IRGRP|S_IROTH)) &&
           (S_ISREG(st.st_mode))) return(0);
    }
    return(0);
}

void enc_sym(int z) {

    char buf[300] = {"\0"};
    if(symg) {

        if(atoi(&gpgver[0]) == 1) return; /* gnupg v.1.x is not supported */
        strlcat(buf, "gpg --trust-model always --yes -q -e -c \
        --cipher-algo ", sizeof(buf));
        strlcat(buf, cipher, sizeof(buf));
        strlcat(buf," --passphrase-repeat 0 --batch --passphrase ",sizeof(buf));
        strlcat(buf, K, sizeof(buf));
        strlcat(buf, " -R ", sizeof(buf));
        strlcat(buf, k[z], sizeof(buf));
        strlcat(buf, " -o ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tcs ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tct", sizeof(buf));
        strlcat(buf, " >/dev/null 2>&1", sizeof(buf));
    }
    if(symo) {

        strlcat(buf, "openssl enc -aes-256-cbc -pbkdf2 -k ", sizeof(buf));
        strlcat(buf, K, sizeof(buf));
        strlcat(buf, " -in ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tct", sizeof(buf));
        strlcat(buf, " -out ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tcs >/dev/null 2>&1", sizeof(buf));
    }
    if(system(buf) == -1) {

        colorwarn(); wprintw(txt, "openssl: %s\n", strerror(errno));
    }
    shred(TCT);
}

void enc_sym_file(int z, char *file) {

    char buf[300] = {"\0"};
    if(symg) {

        if(atoi(&gpgver[0]) == 1) return; /* gnupg v.1.x is not supported */
        strlcat(buf, "gpg --trust-model always --yes -q -e -c \
        --cipher-algo ", sizeof(buf));
        strlcat(buf, cipher, sizeof(buf));
        strlcat(buf," --passphrase-repeat 0 --batch --passphrase ",sizeof(buf));
        strlcat(buf, K, sizeof(buf));
        strlcat(buf, " -R ", sizeof(buf));
        strlcat(buf, k[z], sizeof(buf));
        strlcat(buf, " -o ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tcs ", sizeof(buf));
        strlcat(buf, file, sizeof(buf));
        strlcat(buf, " >/dev/null 2>&1", sizeof(buf));
    }
    if(symo) {

        strlcat(buf, "openssl enc -aes-256-cbc -pbkdf2 -k ", sizeof(buf));
        strlcat(buf, K, sizeof(buf));
        strlcat(buf, " -in ", sizeof(buf));
        strlcat(buf, file, sizeof(buf));
        strlcat(buf, " -out ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tcs >/dev/null 2>&1", sizeof(buf));
    }
    if(system(buf) == -1) {

        colorwarn(); wprintw(txt, "openssl: %s\n", strerror(errno));
    }
    shred(TCT);
}

void enc_asym(int z) {

    if(atoi(&gpgver[0]) == 1) return; /* gnupg v.1.x is not supported */
    char buf[STR] = {"\0"};
    strlcat(buf, "gpg --trust-model always --yes -q -e -R ",sizeof(buf));
    strlcat(buf, k[z], sizeof(buf));
    strlcat(buf, " -o ", sizeof(buf));
    strlcat(buf, home, sizeof(buf));
    strlcat(buf, "/.tc/.tcs ", sizeof(buf));
    strlcat(buf, home, sizeof(buf));
    strlcat(buf, "/.tc/.tct >/dev/null 2>&1", sizeof(buf));
    if(system(buf) == -1) {

        colorwarn(); wprintw(txt, "gpg: %s\n", strerror(errno));
    }
    shred(TCT);
}

void enc_asym_file(int z, char *file) {

    if(atoi(&gpgver[0]) == 1) return; /* gnupg v.1.x is not supported */
    char buf[STR] = {"\0"};
    strlcat(buf, "gpg --trust-model always --yes -q -e -R ",sizeof(buf));
    strlcat(buf, k[z], sizeof(buf));
    strlcat(buf, " -o ", sizeof(buf));
    strlcat(buf, home, sizeof(buf));
    strlcat(buf, "/.tc/.tcs ", sizeof(buf));
    strlcat(buf, file, sizeof(buf));
    strlcat(buf, " >/dev/null 2>&1", sizeof(buf));
    if(system(buf) == -1) {

        colorwarn(); wprintw(txt, "gpg: %s\n", strerror(errno));
    }
    shred(TCT);
}

int ucrypt(char *n, char *ms, char *file) {

    LOCK(&mtxtct)
    if(file == NULL && n != NULL && ms != NULL) { /* NULL file means msg */

        FILE *f = fopen(TCT, "w"); if(f == NULL) return(1);
        fprintf(f, "<%s> %s\n", n, ms);
        fclose(f);
        if(sym) enc_sym(0);
        if(!sym) enc_asym(0);
        sender(o[0], n, 0);
    }
    else {

        if(sym) enc_sym_file(0,file);
        if(!sym) enc_asym_file(0,file);
        sender(o[0], n, 0);
    }
    UNLOCK(&mtxtct)
    return(0);
}

int gcrypt(char *n, char *mg, char *file) {

    LOCK(&mtxtct)
    if(file == NULL && n != NULL && mg != NULL) {

        FILE *f = fopen(TCT, "w"); if(f == NULL) return(1);
        fprintf(f, "<%s> %s\n", n, mg);
        fclose(f);
        if(sym) {

            enc_sym(0);
            for(int i = 0; i < who; i++) sender(o[i], n, 0);
        }
        if(!sym) {

            for(int i = 0; i < who; i++) {

                enc_asym(i); sender(o[i], n, 0);
            }
        }
    }
    else {

        if(sym) {

            enc_sym_file(0,file);
            for(int i = 0; i < who; i++) sender(o[i], n, 0);
        }
        if(!sym) {

            for(int i = 0; i < who; i++) {

                enc_asym_file(i,file); sender(o[i], n, 0);
            }
        }
    }
    UNLOCK(&mtxtct)
    return(0);
}

void forward(void) {

    (grp) ? gcrypt("*", msg, NULL) : ucrypt("*", msg, NULL);
}

void sendinfos(char *ss) {

    char buf[STR] = {"\0"};
    strlcat(buf, nick, sizeof(buf));
    strlcat(buf, ss, sizeof(buf));
    strlcpy(msg, buf, sizeof(msg));
    forward();
}

void decrypt(int sig) {

    char buf[STR] = {"\0"};
    if(sym && symo) {

        strlcat(buf, "openssl enc -aes-256-cbc -pbkdf2 -k ",sizeof(buf));
        strlcat(buf, K, sizeof(buf));
        strlcat(buf, " -in ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tcr", sizeof(buf));
        strlcat(buf, " -out ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tcn -d >/dev/null 2>&1", sizeof(buf));
    }
    if(sym && symg) {

        strlcat(buf,
        "gpg --pinentry-mode loopback --yes -q --batch --passphrase ",
        sizeof(buf));
        strlcat(buf, K, sizeof(buf));
        strlcat(buf, " -o ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tcn ", sizeof(buf));
        strlcat(buf, " -d ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tcr >/dev/null 2>&1", sizeof(buf));
    }
    if(!sym && !symg && !symo) {

        strlcat(buf,
        "gpg --pinentry-mode loopback --yes -q --batch --passphrase ",
        sizeof(buf));
        strlcat(buf, PASS, sizeof(buf));
        strlcat(buf, " -o ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tcn ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/.tcr >/dev/null 2>&1", sizeof(buf));
    }
    if(!access(TCR, F_OK)) {

        if(system(buf) == -1) {

            colorwarn(); wprintw(txt, "gpg decrypt: %s\n", strerror(errno));
        }
    }
    unlink(TCR);
    FILE *f = fopen(TCN, "r");
    if(f == NULL) return;
    char rem[MSG] = {"\0"};
    if(fgets(rem, MSG, f) == NULL) *rem = '\0';
    rem[strcspn(rem, "\n")] = '\0';
    if(strstr(rem, "<decoy>") != NULL) {

        fclose(f); shred(TCN); return;
    }
    REPOSITION(f)
    long long siz = ftell(f);
    if(siz > MSG) { /* assume as file */

        char fn[STR];
        int bn = 0;
        strlcpy(fn, tcrcvfile, sizeof(fn));
        if((bn = mkstemp(fn))) { rename(TCN, fn); fclose(f); }
        else { unlink(fn); close(bn); superbad(); }
        char skaled[FMT];
        strlcpy(skaled, ska(siz), sizeof(skaled));
        colorinfo();
        wprintw(txt, "file recieved: %s (%s)\n", fn, skaled);
        char b[STR] = {"\0"};
        strlcat(b, "<*> file recieved: ", sizeof(b));
        strlcat(b, fn, sizeof(b));
        strlcat(b, " (", sizeof(b));
        strlcat(b, skaled, sizeof(b));
        strlcat(b, ")", sizeof(b));
        addscrollback(b);
        if(logs) logger(NULL, b);
        shred(TCN);
    }
    else { /* its a msg; tiny files are displayed in-line */

        getyx(mainwin, y, x);
        if(rem[0] != '\0') drawremote(rem);
        fclose(f);
        shred(TCN);
    }
    /* notify away if joined and not idle, avoid ping-pong away notice */
    if(con && away && !grp) {

        if(strstr(rem, " is away: ") != NULL) return;
        else {

            char re[MSG] = {"\0"};
            strlcat(re, " is away: ", sizeof(re));
            strlcat(re, way, sizeof(re));
            sendinfos(re);
        }
    }
    /* prevent blank line in symmetric if someone tcp CONNECT to tc daemon */
    if(strstr(rem, " activated symmetric ") != NULL)
        noblank = 1;
    if(strstr(rem, " has switched OFF symmetric encyption") != NULL)
        noblank = 0;
    return;
}

void showcontacts(char *nam) {

    if(nam == NULL) {

        hm(); c_info(); wprintw(txt, "<*> contacts:"); c_info_off();
        for(int i = 0; i < nusr; i++) {

            if(i%6 == 0) {

                wprintw(txt, "\n"); hm(); wprintw(txt, " ");
            }
            wprintw(txt, "[");
            c_whit(); wprintw(txt, "%-9s", c[i].name); c_whit_off();
            wprintw(txt, "] ");
        }
        wprintw(txt, "\n"); hm(); wprintw(txt, "(total count: %d)\n", nusr);
    }
    else {

        int found = 0;
        for(int i = 0; i < nusr; i++) {

            if(!strcmp(c[i].name, nam)) {

                hm();
                c_info(); wprintw(txt, "<*> %s:\n", c[i].name); c_info_off();
                hm(); wprintw(txt, "keyid: ");
                c_whit(); wprintw(txt, "%s\n", c[i].key); c_whit_off();
                hm(); wprintw(txt, "onion: ");
                c_warn(); wprintw(txt, "%s\n", c[i].onion); c_warn_off();
                found = 1;
            }
        }
        if(!found) {

            colorwarn(); wprintw(txt, "%s not found in contacts\n", nam);
        }
    }
}

void showgroups(void) {

    hm(); c_info(); wprintw(txt, "<*> groups:"); c_info_off();
    for(int i = 0; i < ngrp; i++) {

        wprintw(txt, "\n");
        hm(); c_whit(); wprintw(txt, "%-13s", g[i].grpname); c_whit_off();
        for(int q = 0; q < MAX_MEMBERS; q++) {

            wprintw(txt, "[");
            c_warn(); wprintw(txt, "%-9s", (char *)&g[i].members[q]);
            c_warn_off(); wprintw(txt, "] ");
        }
    }
    wprintw(txt, "\n"); hm(); wprintw(txt, "(total group: %d)\n", ngrp);
}

void history(char *mex) {

    if(mex == NULL) {

        pager = 1;
        int pos = row-3, l = 0;
        hm(); c_info(); wprintw(txt, "<*> history:\n"); c_info_off();
        for(int i = 0; i < hpos; i++) {

            if(l++ ==  pos) {

                getch(); l = 1; for(int j = 1; j < pos; j++) clrtoeol();
            }
            hm(); wprintw(txt, "[%d] %s\n", i, s[i]);
        }
        pager = 0;
    }
    else {

        strlcpy(s[hpos], mex, MSG); hpos++;
        if(hpos == MSG) { memset(s, 0, sizeof(s)); hpos = 0; }
    }
}

int checkkey(char *nam, char *ck) {

    int keyok = 0;
    for(int i = 0; i < nkey; i++) {

        if(ck == NULL) break; if(!strcmp(ring[i].keyid, ck)) keyok = 1;
    }
    if(keyok) return(0);
    else {

        colorwarn();
        wprintw(txt, "%s keyid %s not found in local PGP key ring\n", nam, ck);
        return(1);
    }
}

int loadu(char *n) {

    int keynok = 0, found = 0;
    for(int i = 0; i < nusr; i++) {

        if(!strcmp(c[i].name, n)) {

            if(!checkkey(c[i].name, c[i].key)) {

                found = 1;
                k[who] = c[i].key;
                o[who] = c[i].onion;
                strlcpy(remo, n, sizeof(remo));
                who++;
            }
            else keynok = 1;
        }
    }
    if(!found) {

        colorwarn(); wprintw(txt, "%s not found in contacts\n", n);
    }
    if(keynok || !found) return(1);
    return(0);
}

int loadg(char *n) {

    for(int i = 0; i < ngrp; i++) {

        if(!strcmp(g[i].grpname, n)) {

            for(int q = 0; q < MAX_MEMBERS; q++) {

                if(!strcmp(g[i].members[q], "\n")) break;
                if(!strcmp(g[i].members[q], "")) break;
                if(!loadu(g[i].members[q])) {

                    grp = 1; strlcpy(remo, g[i].grpname, sizeof(remo));
                }
            }
        }
    }
    if(!grp) return(1);
    return(0);
}

int checkname(char *n) {

    int uu = 0, gg = 0;
    for(int j = 0; j <= nusr+1; j++) { if(!strcmp(c[j].name, n)) uu++; }
    for(int j = 0; j <= ngrp+1; j++) { if(!strcmp(g[j].grpname, n)) gg++; }
    if(!uu && !gg) {

        colorwarn(); wprintw(txt,"%s not found in contacts\n", n); return(1);
    }
    if(uu > 1 ) {

        colorwarn();
        wprintw(txt, "too many entry found for \"%s\" in contacts\n", n);
        return(1);
    }
    if(gg > 1 ) {

        colorwarn();
        wprintw(txt, "too many entry found for \"%s\" in groups\n", n);
        return(1);
    }
    if(uu && gg) {

        char e[2] = {"\0"};
        prompy("[*] too many entry; is (c)ontact or (g)roup? [c/g]: ");
        GETSTRING(e, 1);
        if(e[0] == 'c')      { if(loadu(n)) return(1); }
        else if(e[0] == 'g') { if(loadg(n)) return(1); }
        else return(1);
    }
    if(uu && !gg)   { if(loadu(n)) return(1); }
    if(!uu && gg)   { if(loadg(n)) return(1); }
    return(0);
}

int saveonion(void) {

    if(!access(tchost, F_OK)) {

        char *b = hidden();
        char current[64] = {"\0"};
        strlcat(current, b, sizeof(current)); free(b);
        hm(); wprintw(txt, "shutting: %s\n", current);
        if(killtor()) return(1);
        if(con) nocom();
        draw();
        char buf[STR] = {"\0"};
        strlcat(buf, "mv ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/tor ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/tor.", sizeof(buf));
        strlcat(buf, current, sizeof(buf));
        if(system(buf) == -1) {

            colorwarn(); wprintw(txt, "mv: %s\n", strerror(errno));
        }
        return(0);
    }
    colorwarn(); wprintw(txt, "nothing to save\n");
    return(1);
}

int curlo(char *host, int sox) {

    if (sox) { sender(host, nick, 1); return 0; }

    struct addrinfo hints, *res, *res0;
    int stst = -1;
    memset(&hints, 0, sizeof (hints));
    hints.ai_family = PF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    if(getaddrinfo(host, "80", &hints, &res0) != 0) return(1);
    for(res = res0; res; res = res->ai_next) {
        stst = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
        if(stst < 0) return(1);
        if(connect(stst, res->ai_addr, res->ai_addrlen) < 0) return(1);
        break;
    }
    if (stst != -1 && sox)  get_page(stst, host, 1);
    if (stst != -1 && !sox) get_page(stst, host, 0);
    freeaddrinfo(res0);
    return(0);
}

int alive(void) {

    theylive = 1;
    char *host = "checkip.dyndns.org";
    curlo(host, 0); fireuptor(); curlo(host, 1); theylive = 0;
    if(chk[0] == '\0' || chktor[0] == '\0') return(0); else return(1);
}

void *clocker(void *arg) {

    while (1) {

        TIMESCAPE
        torpid();
        getyx(mainwin, y, x); draw(); move(y, x);
        if(flood && (nmex > burst)) nmex = 0;
        if(!tm.tm_hour && !tm.tm_min && !tm.tm_sec) {

            colorinfo(); wprintw(txt, "day changed to %s", ctime(&t));
        }
        /* check tor process and ip addresses */
        elapse(tkep); if(con && undead != 0 && ((min-undead) == 0)) {

            if(!alive()) {

                undead = 0; tkep = 0;
            }
            else  tkep = time(&t);
        }
        /* remote messages counter reset after 2 minutes when antiflood */
        elapse(tflo); if(con && flood && (min % 2) == 0) nmex = 0;

        sleep(1);
    }
}

void *send_decoy(void *arg) {

    while(con) {

        int deception = lrand48() % 120;
        if(ndec > 8) { /* burst minimum is 10 */

            sleep(90); ndec = 0; continue;
        }
        sleep(deception);
        int fake = lrand48() % 7; /* minor of MSG */
        int i = 1;
        char buf[MSG] = {"\0"};
        while (i <= fake) {

            const char *b = mkfake(0);
            if(b != NULL) { strlcat(buf, b, sizeof(buf)); i++; }
            free((void *)b);
        }
        (grp) ? gcrypt("decoy",buf,NULL) : ucrypt("decoy",buf,NULL);
        ndec++;
    }
    return NULL;
}

void *listener(void *arg) {

    int sockfd = 0;
    int nsockfd = 0;
    struct sockaddr_in local;   /* tc server */
    struct sockaddr_in remote;  /* client address */
    memset(&local, 0, sizeof(local)); /* clear struct */
    memset(&remote, 0, sizeof(remote)); /* clear struct */
    char revbuf[RWLENGTH];
    if((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) exit(1);
    local.sin_family = AF_INET;
    local.sin_port = htons(TCPORT);
    inet_pton(AF_INET, TCDAEMON, &local.sin_addr);
    memset(&local.sin_zero, 0, sizeof(local.sin_zero)); /* clear struct */
    int reuse = 1;
    struct timeval tv;
    tv.tv_sec = ALIVE;
    tv.tv_usec = 0;
    setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv));
    if(setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (const char*)&reuse,
    sizeof(reuse)) < 0) exit(1);
    if(setsockopt(sockfd, SOL_SOCKET, SO_REUSEPORT, (const char*)&reuse,
    sizeof(reuse)) < 0) exit(1);
    if(bind(sockfd, (struct sockaddr*)&local, sizeof(struct sockaddr)) == -1)
    superbad();
    if(listen(sockfd, BACKLOG) == -1)
    superbad();
    while(1) {

        if(flood && (nmex > burst)) continue; /* antiflood */
        if(!lazy) {

            unsigned int sin_size = sizeof(struct sockaddr_in);
            if((nsockfd = accept(sockfd, (struct sockaddr *)&remote,
            &sin_size)) == -1) exit(1);
        }
        else {

            close(nsockfd); continue;
        }
        LOCK(&mtxtcr)
        FILE *fr = fopen(TCR, "w");
        if(flood) nmex++;
        if(fr == NULL) errx(1, "NULL .tcr\n");
        else {

            memset(revbuf, 0, sizeof(revbuf));
            int block = 0;
            while((block = recv(nsockfd, revbuf, RWLENGTH, MSG_WAITALL)) > 0) {

                int write = fwrite(revbuf, sizeof(char), block, fr);
                if(write < block) errx(1, "failed write on server");
                memset(revbuf, 0, sizeof(revbuf));
                if(!block || block != RWLENGTH) break;
            }
            if(block < 0) errx(1, "block size under 0\n");
            REPOSITION(fr)
            long long flen = ftell(fr);
            bytein =  bytein + flen;
            /* wprintw(txt, "remote IP: %s.\n", inet_ntoa(remote.sin_addr)); */
            fclose(fr);
            int pid = getpid();
            kill(pid, SIGUSR2);
        }
        UNLOCK(&mtxtcr)
        close(nsockfd);
    }
    return NULL;
}

void keyboard(void) {

    if(nrow  >= 30000) nrow = 0;
    int h = hpos, ccc = -1, visible, pos = nrow;
    for(int i = 0; i <= IN; ++i) {

        if(scrollback) {

            wrefresh(stdscr); redrawwin(txt);
        }
        getyx(mainwin, y, x);
        int cx = getch();
        if(cx == '\n') break;
        scrollback = 0;
        switch(cx) {

        case KEY_PPAGE:

            wrefresh(stdscr);
            getyx(mainwin, y, x);
            getmaxyx(mainwin, row, col);
            visible = row-3;
            if(nrow > 0) {

                scrollback = 1;
                if(pos <= 0) {

                    pos = 0;
                    move(y, x);
                    i = strlen(in);
                    break;
                }
             pos = pos - (visible/2);
             prefresh(pad, pos, 0, 1, 0, visible, (COLS-1));
            }
            move(y, x);
            i = strlen(in);
        break;
        case KEY_NPAGE:

            wrefresh(stdscr);
            getyx(mainwin, y, x);
            getmaxyx(mainwin, row, col);
            visible = row-3;
            if(nrow > 0) {

                scrollback = 1;
                if(pos >= nrow || pos + visible >= nrow) {

                    pos = nrow - visible;
                    prefresh(pad, pos, 0, 1, 0, visible, (COLS-1));
                    move(y, x);
                    i = strlen(in);
                    break;
                }
                pos = pos + (visible/2);
                prefresh(pad, pos, 0, 1, 0, visible, (COLS-1));
            }
            move(y, x);
            i = strlen(in);
        break;
        case KEY_LEFT:

            getmaxyx(mainwin, row, col);
            if(ccc == -1) { ccc = (COMMANDS-1); }
            if(ccc < 0) {

                deleteln();
                in[0] = '\0';
                msg[0] = '\0';
                return;
            }
            deleteln();
            if(!ccc) { ccc = (COMMANDS-1); }
            if(ccc != 0) ccc--;
            memcpy(in, commands[ccc], sizeof(in));
            mvprintw((row-1), 0, " =--> %s", commands[ccc]);
            move((row-1), (6+strlen(commands[ccc])));
            getyx(mainwin, y, x);
            move(y, x);
            i = strlen(in);
        break;
        case KEY_RIGHT:

            getmaxyx(mainwin, row, col);
            if(ccc > (COMMANDS-1)) {

                deleteln();
                in[0] = '\0';
                msg[0] = '\0';
                return;
            }
            deleteln();
            if(ccc != (COMMANDS-1)) ccc++;
            if(ccc == (COMMANDS-1)) ccc = 0;
            memcpy(in, commands[ccc], sizeof(in));
            mvprintw((row-1), 0, " =--> %s", commands[ccc]);
            move((row-1), (6+strlen(commands[ccc])));
            getyx(mainwin, y, x);
            move(y, x);
            i = strlen(in);
        break;
        case KEY_UP: /* don't work on vt100/vt220 */

            getmaxyx(mainwin, row, col);
            if(h < 0) {

                in[0] = '\0';
                msg[0] = '\0';
                return;
            }
            deleteln();
            if(h != 0) h--;
            memcpy(in, s[h], 61);
            in[IN] = '\0';
            mvprintw((row-1), 0, " =--> %s", in);
            move((row-1), (6+strlen(in)));
            getyx(mainwin, y, x);
            move(y, x);
            i = strlen(in);
        break;
        case KEY_DOWN: /* don't work on vt100/vt220 */

            getmaxyx(mainwin, row, col);
            if(h > hpos) {

                in[0] = '\0';
                msg[0] = '\0';
                return;
            }
            deleteln();
            if(h != hpos) h++;
            memcpy(in, s[h], 61);
            in[IN] = '\0';
            mvprintw((row-1), 0, " =--> %s", in);
            move((row-1), (6+strlen(in)));
            getyx(mainwin, y, x);
            move(y, x);
            i = strlen(in);
        break;
        case 127:                                  /* FALLTHROUGH */
        case KEY_BACKSPACE:
        case KEY_CANCEL:
        case '\b':
        case KEY_DC:

            if(x >= 7) {                   /* don't delete prompt */

                getyx(mainwin, y, x);
                move(y, x-1);
                delch();
                in[i] = 0;
                in[i+1] = 0;
                in[i-1] = 0;
            }
            else {

                x = 7;
            }
            if(i >= 0) {         /* overwrite already typed chars */

                i = i - 1;
            }
            if(i < 0) {    /* don't break on backspaces underflow */

                i = 0;
                break;
            }
        break;
        case CTRL('c'):

            cleanup(0);

        default: break;
    }
    if(!isprint(cx)) i--;
    if(isprint(cx)) {

        if(cx == KEY_PPAGE || cx == KEY_NPAGE || cx == KEY_END ||
           cx == KEY_CANCEL || (i == IN && cx != '\n')) i--;
        else {

            in[i] = cx; in[i+1] = '\0'; addch(cx);
        }
    }
    } /* for loop end here*/
    strlcpy(msg, in, IN+1); scrollback = 0; deleteln();
}

void createtcdirs(void) {

    FILE *d = NULL;
    d = fopen(tcdir, "r"); if(d == NULL)
    errx(1,"tc's dir not found: \"mkdir %s\" and restart", tcdir);
    else fclose(d);

    d = fopen(tctordir, "r"); if(d == NULL) {
        if(mkdir((const char *)tctordir, 0700) != 0)
        fprintf(stderr, "error: %d making %s\n", errno, tctordir);
    } else fclose(d);

    d = fopen(tcauthdir, "r"); if(d == NULL) {
        if(mkdir((const char *)tcauthdir, 0700) != 0)
        fprintf(stderr, "error: %d making %s\n", errno, tcauthdir);
    } else fclose(d);

    d = fopen(tchiddendir, "r"); if(d == NULL) {
        if(mkdir((const char *)tchiddendir, 0700) != 0)
        fprintf(stderr, "error: %d making %s\n", errno, tchiddendir);
    } else fclose(d);

    d = fopen(tcauthclidir, "r"); if(d == NULL) {
        if(mkdir((const char *)tcauthclidir, 0700) != 0)
        fprintf(stderr, "error: %d making %s\n", errno, tcauthclidir);
    } else fclose(d);
}

int pleasedontoverwriteme(char *filename) {

    char buf[STR] = {"\0"};
    strlcat(buf, "ls ", sizeof(buf));
    strlcat(buf, filename, sizeof(buf));
    strlcat(buf, " 2>&1", sizeof(buf));
    FILE *p = NULL;
    if((p = popen(buf, "r")) == NULL) return(1);
    else {

        char f[STR] = {"\0"};
        fgets(f, STR, p); f[strcspn(f,"\n")] = '\0';
        BADPCLOSE(p)
        if(!strcmp(f, filename)) {

            colorwarn(); wprintw(txt, "keyname already in use\n"); return(1);
        }
        if(strstr(filename, "@") != NULL) {

            filename[strcspn(filename,"@")] = '\0';
            if(pleasedontoverwriteme(filename) == 1) return(1);
        }
    }
    return(0);
}

int keygen(void) {

    prompy("[*] generate new authorization keys? [y/n]: "); GETSTRING(msg, 1);
    if(msg[0] != 'y') return(1);
    else {

        char kname[N+5];
        prompy("[*] keys name: "); GETSTRING(kname, N);
        if(kname[0] == '\0') return(1);
        deblank(kname);
        char const lnxprv[] = {
        " | grep -v \" PRIVATE KEY\" | base64pem -d | tail --bytes=32 | base32|\
          sed 's/=//g' >> "};
        char const lnxpub[] = {
        " -pubout | grep -v \" PUBLIC KEY\" | base64pem -d | tail --bytes=32 |\
          base32 | sed 's/=//g' >> "};
        char const bsdprv[] = {
        " | grep -v \" PRIVATE KEY\" | gbase64 -d -w 64 | gtail --bytes=32 |\
          gbase32 | sed 's/=//g' >> "};
        char const bsdpub[] = {
        " -pubout | grep -v \" PUBLIC KEY\" | gbase64 -d -w 64 |\
          gtail --bytes=32 | gbase32 | sed 's/=//g' >> "};
        FILE *p = NULL;
        if((p = popen("uname", "r")) == NULL) return(1);
        char os[N+5] = {"\0"};
        fgets(os, N, p); os[strcspn(os, "\n")] = '\0';
        BADPCLOSE(p)
        createtcdirs();
        char buf[STR] = {"\0"};
        strlcat(buf, "openssl genpkey -algorithm x25519 -out ", sizeof(buf));
        strlcat(buf, pem, sizeof(buf));
        strlcat(buf, " >/dev/null 2>&1", sizeof(buf));
        p = NULL;
        if((p = popen(buf, "r")) == NULL) return(1);
        BADPCLOSE(p)
        char kprv[STR] = {"\0"}; /* private key */
        strlcat(kprv, tcauthdir, sizeof(kprv));
        strlcat(kprv, "/", sizeof(kprv));
        strlcat(kprv, kname, sizeof(kprv));
        strlcat(kprv, ".auth_private@", sizeof(kprv));
        if(pleasedontoverwriteme(kprv) == 1) return(1);
        char kpub[STR] = {"\0"}; /* public key */
        strlcat(kpub, tcauthclidir, sizeof(kpub));
        strlcat(kpub, "/", sizeof(kpub));
        strlcat(kpub, kname, sizeof(kpub));
        strlcat(kpub, ".auth@", sizeof(kpub));
        if(pleasedontoverwriteme(kpub) == 1) return(1);
        char *oa = getonion();
        if(oa == NULL) return(1);
        FILE *f1 = NULL;
        if((f1 = fopen(kprv, "w")) == NULL) { free(oa); return(1); }
        else {

            char nodotonion[57] = {"\0"};
            strlcpy(nodotonion, oa, sizeof(nodotonion));
            fprintf(f1, "%s", nodotonion);
            fprintf(f1, ":descriptor:x25519:");
            fclose(f1);
        }
        FILE *t1 = fopen(pem, "r");
        if(t1 == NULL) {

            colorwarn(); wprintw(txt, "no certificate found\n");
            shred(kprv); return(1);
        }
        else fclose(t1);
        char b1[STR*2] = {"\0"};
        strlcat(b1, "cat ", sizeof(b1));
        strlcat(b1, pem, sizeof(b1));
        if(!strcmp(os, "Linux")) strlcat(b1, lnxprv, sizeof(b1));
        else                     strlcat(b1, bsdprv, sizeof(b1));
        strlcat(b1, kprv, sizeof(b1));
        FILE *p1 = NULL;
        if((p1 = popen(b1, "r")) == NULL) return(1);
        BADPCLOSE(p1)
        FILE *f2 = NULL;
        if((f2 = fopen(kpub, "w")) == NULL) return(1);
        else {

            fprintf(f2, "descriptor:x25519:");
            fclose(f2);
        }
        FILE *t2 = fopen(pem, "r");
        if(t2 == NULL) {

            colorwarn(); wprintw(txt, "no certificate found\n");
            shred(kpub); return(1);
        }
        else fclose(t2);
        char b2[STR*2] = {"\0"};
        strlcat(b2, "openssl pkey -in ", sizeof(b2));
        strlcat(b2, pem, sizeof(b2));
        if(!strcmp(os, "Linux")) strlcat(b2, lnxpub, sizeof(b2));
        else                     strlcat(b2, bsdpub, sizeof(b2));
        strlcat(b2, kpub, sizeof(b2));
        FILE *p2 = NULL;
        if((p2 = popen(b2, "r")) == NULL) return(1);
        BADPCLOSE(p2)
    }
    ibox = 1;
    INFOBOX
    i_info(); mvwprintw(info,1,23,"notice"); i_info_off();
    mvwprintw(info,4,5,"Give the .auth file to clients who need access.");
    mvwprintw(info,5,5,"If you generate keys not for local onion edit");
    mvwprintw(info,6,5,"hostname in .auth_private to match the real.");
    mvwprintw(info,7,5,"Tor must be restarted after each key operation.");
    mvwprintw(info,8,5,"Each file in $HOME/.tc/tor/auth/ must contain");
    mvwprintw(info,9,5,"different hostnames. Revoke client access to");
    mvwprintw(info,10,5,"your site removing the .auth file from");
    mvwprintw(info,11,5,"$HOME/.tc/tor/hidden_service/authorized_clients/");
    wrefresh(info); draw(); prompy(""); noecho(); getch(); delwin(info);
    ibox = 0;
    shred(pem);
    return(0);
}

void printkeys(void) {

    hm(); c_info(); wprintw(txt,"<*> pgp keys:\n"); c_info_off();
    for(int i = 0; i < nkey; i++) {

        hm(); wprintw(txt,"  %d.[ ", i);
        ring[i].descr[strcspn(ring[i].descr,"\n")] = '\0';
        c_whit(); wprintw(txt,"%-64s", ring[i].descr); c_whit_off();
        wprintw(txt," ]\n");
    }
    hm(); wprintw(txt,"(total count: %d)\n",nkey);
    loadauthkeys(); printkauth(); printkprv(); draw(); keygen();
}

void xtermresize(int sig) { /* XXX if r > 3 blank r-3 lines before bottom txt */

    endwin(); refresh(); move(row-1, 0); clrtoeol(); move(row-2, 0); clrtoeol();
    int new_row;
    getmaxyx(mainwin, new_row, col);
    getyx(txt, y, x);
    if(new_row < row && y >= new_row-2) {

        int r = row - new_row;
        wscrl(txt, r); refresh();
    }
    row = new_row;
    wresize(mainwin, row, col);
    wresize(txt, row-2, col);
    refresh(); draw(); redrawwin(txt); refresh();
}

int torcircuit(void) {

    waitingfor();
    hm(); c_info(); wprintw(txt, "<*> circuit:\n"); c_info_off();
    alive();
    hm(); wprintw(txt, "real: ");
    c_whit(); wprintw(txt, "%s", chk); c_whit_off();
    wprintw(txt, " exit node: ");
    c_whit(); wprintw(txt, "%s\n", chktor); c_whit_off();
    char *b = hidden();
    if(b != NULL) { hm(); wprintw(txt, "host: %s\n", b); free(b); }
    return(0);
}

void createrc(void) {

    FILE *f = fopen(tcrc, "w");
    if(f == NULL) errx(1,"createrc");
    char buf[MSG] = {"\0"};
    strlcat(buf, home, sizeof(buf));
    strlcat(buf, "/.tc", sizeof(buf));
    fprintf(f, "Log notice syslog\n");
    fprintf(f, "RunAsDaemon 1\n");
    fprintf(f, "DataDirectory %s/tor\n", buf);
    fprintf(f, "HiddenServiceDir %s/tor/hidden_service\n", buf);
    fprintf(f, "HiddenServicePort 2323 127.0.0.1:2323\n");
    fprintf(f, "ClientOnionAuthDir %s/tor/auth\n", buf);
    fclose(f);
}

void createtcconf(void) {

    FILE *f = fopen(tcconf, "w");
    if(f == NULL) errx(1,"createtcconf");
    fprintf(f, "# passphrase\n");
    fprintf(f, "''\n");
    fprintf(f, "# Tor proxy address and port\n");
    fprintf(f, "127.0.0.1:9050\n");
    fprintf(f, "# tc address and port\n");
    fprintf(f, "127.0.0.1:2323\n");
    fprintf(f, "# colors\n");
    fprintf(f, "2:0\n");
    fprintf(f, "7:4\n");
    fprintf(f, "7:0\n");
    fprintf(f, "5:0\n");
    fprintf(f, "# contacts\n");
    fprintf(f, "tc:84BE7F4663CE2B5A:%s\n",TCURL);
    fprintf(f, "# groups\n");
    fprintf(f, "Ramones:Joey,DeeDee,Johnny,Tommy,\n");
    fprintf(f, "JAZZ:Blakey,Gillespie,Telonious,Davis,Parker,\n");
    fprintf(f, "# eof\n");
    fclose(f);
}

void invalidnick(char *st) {

    char bad[] = "badnick";
    if(strstr(st, "à") != NULL) strlcpy(nick, bad, sizeof(nick));
    if(strstr(st, "è") != NULL) strlcpy(nick, bad, sizeof(nick));
    if(strstr(st, "ì") != NULL) strlcpy(nick, bad, sizeof(nick));
    if(strstr(st, "ò") != NULL) strlcpy(nick, bad, sizeof(nick));
    if(strstr(st, "ù") != NULL) strlcpy(nick, bad, sizeof(nick));
    if(strstr(st, "<*>") != NULL) strlcpy(nick, bad, sizeof(nick));
    if(strstr(st, "<*") != NULL) strlcpy(nick, bad, sizeof(nick));
    if(strstr(st, "*>") != NULL) strlcpy(nick, bad, sizeof(nick));
    if(strstr(st, "<decoy>") != NULL) strlcpy(nick, bad, sizeof(nick));
    if(!strcmp(st, "decoy")) strlcpy(nick, bad, sizeof(nick));
    if(!strcmp(st, "*")) strlcpy(nick, bad, sizeof(nick));
}

char *mkrandom(void) {

    char *otp = (char *) calloc(1, 16);
    if(otp == NULL) { free(otp); return NULL; }
    FILE *p = NULL;
    if((p = popen("gpg --armor --gen-random 0 16", "r")) == NULL) {

        free(otp); return NULL;
    }
    fgets(otp, N+6, p);
    otp[strcspn(otp, "\n")] = '\0';
    BADPCLOSE(p)
    deblank(otp);
    return otp;
}

void setsympass(void) {

m00t:
    prompy("[*] symmetric encrypt with (g)pg or (o)penssl: "); GETSTRING(msg,1);
    switch((int)msg[0]) {

        case 'g': symg = 1; break;
        case 'o': symo = 1; break;
        default: goto m00t;
    }
w00t:
    prompy("[*] (s)et password or use (o)tp: "); GETSTRING(msg, 1);
    char *otp = NULL;
    switch((int)msg[0]) {

        case 's':
            prompy("[*] set passphrase: "); GETSTRING(msg, N+5);
            otp = deblank(msg);
            memset(K, 0, sizeof(K));
            strlcpy(K, otp, sizeof(K));
            if(msg[0] == '\0') goto w00t;
        break;
        case 'o':
            memset(K, 0, sizeof(K));
            otp = mkrandom();
            strlcpy(K, otp, sizeof(K));
        break;
        default: goto w00t;
    }
    if(symg) {

        colorinfo();
        wprintw(txt, "symmetric encryption: ON (gpg key: %s)\n", otp);
    }
    if(symo) {

        colorinfo();
        wprintw(txt, "symmetric encryption: ON (openssl key: %s)\n", otp);
    }
    if(con && !noblank) {

        char buf[STR] = " activated symmetric ";
        if(symg)
        strlcat(buf, "GPG", sizeof(buf));
        if(symo)
        strlcat(buf, "OPENSSL", sizeof(buf));
        strlcat(buf, " with key: ", sizeof(buf));
        strlcat(buf, otp, sizeof(buf));
        sendinfos(buf);
    }
}

void algoz(void) {

    hm(); c_info(); wprintw(txt, "<*> symmetric cipher algos:"); c_info_off();
    for(int i = 0; i < (SKA-1); i++) {

        if(i%3 == 0) {

            wprintw(txt,"\n"); hm();
        }
        wprintw(txt,"  %d.[ ", i);
        if(!strcmp(cipher, alg[i])) c_whit();
        wprintw(txt, "%-11s", alg[i]); c_whit_off();
        wprintw(txt," ]");
    }
    wprintw(txt, "\n"); hm(); wprintw(txt, "(total count: %d)\n", (SKA-1));
    draw();
    if(con && sym && symg) {

        prompy("[*] choose cipher: "); GETSTRING(msg, 1);
        if(msg[0] == '\0' || !isdigit(msg[0])) return;
        int on = (atoi(msg));
        if(on >= 0 && on < (SKA-1)) {

            memset(cipher, 0, sizeof(cipher));
            strlcpy(cipher, alg[on], sizeof(cipher));
            colorinfo();
            wprintw(txt, "encryption cipher switched to %s\n", cipher); draw();
            char buf[MSG] = {"\0"};
            strlcat(buf, " has switched cipher to: ", sizeof(buf));
            strlcat(buf, cipher, sizeof(buf));
            sendinfos(buf);
        }
    }
}

void awayz(void) {

    if(!away) {

        prompy("[*] set away message: "); GETSTRING(way, N*4);
        if(way[0] == '\0') { strlcpy(way, " ", 1); }
        colorinfo(); wprintw(txt, "you have been marked as being away\n");
        away = 1; logs = 1;
    }
    else {

        colorinfo(); wprintw(txt, "you are no longer marked as being away\n");
        away = 0; logs = 0;
    }
}

int botz(void) {

    waitingfor();
    FILE *b = NULL;
    b = fopen(tcbotsh, "r");
    if(b != NULL) {

        fclose(b);
        char buf[STR] = {"\0"};
        strlcat(buf, tcbotsh, sizeof(buf));
        strlcat(buf, " >/dev/null 2>&1", sizeof(buf));
        if(system(buf) == -1) {

            colorwarn(); wprintw(txt, "bot: %s\n", strerror(errno));
        }
        if(showfile(NULL)) {

            colorwarn(); wprintw(txt, ".bot output not found\n");
        }
    }
    else { colorwarn(); wprintw(txt, "bot.sh file not found\n"); }
    return(0);
}

int callz(void) {

    if(!con) {

        char *oa = getonion();
        if(oa == NULL) return(1);
        strlcpy(callo, oa, sizeof(callo));
        free(oa);
        o[0] = callo;
        msg[0] = '\0';
        if(sym) setsympass();
        prompy("[*] pgp keyid: "); GETSTRING(msg, 17-1);
        if(msg[0] == '\0' || strlen(msg) != 16 || checkkey("",msg) != 0) {

            colorwarn(); wprintw(txt, "invalid pgp key\n");
            return(1);
        }
        msg[strcspn(msg, "\n")] = '\0';
        strlcpy(remo, callo, sizeof(remo));
        strlcpy(callk, msg, sizeof(callk));
        k[0] = callk;
        colorinfo(); wprintw(txt, "joining %s", o[0]);
        waitingfor(); who++; con = 1;
        char buf[] = " has joined the conversation";
        sendinfos(buf);
    }
    else { colorwarn(); wprintw(txt, "already joined to %s\n", remo); }
    return(0);
}

int camuflagez(void) {

    if(!camuflage) {

        colorinfo(); wprintw(txt, "camuflage: ON\n"); camuflage = 1;
    }
    else {

        colorinfo(); wprintw(txt, "camuflage: OFF\n"); camuflage = 0;
    }
    return(0);
}

void clearz(void) {

    clear(); delwin(txt);
    if(ibox) delwin(info);
    getmaxyx(mainwin, row, col);
    txt = newwin(row-2, col, 1, 0);
    scrollok(txt, TRUE);
}

void colorz(void) {

    hm(); c_info(); wprintw(txt, "<*> colors:"); c_info_off();
    for(int i = 0; i < (9-1); i++) {

        if(i%4 == 0) {

            wprintw(txt,"\n"); hm();
        }
        wprintw(txt,"  %d.[ ", i);
        wprintw(txt, "%-7s", colors[i]); c_whit_off();
        wprintw(txt," ]");
    }
    wprintw(txt, "\n"); hm(); wprintw(txt, "(total count: %d)\n", (9-1));
    draw();
    int i = -1;
    prompy("[*] set info color: "); GETSTRING(msg, 1);
    if(msg[0] != '\0' && isdigit(msg[0])) i = atoi(msg);
    if(i >= 0 && i <= 7) {

        info_f = i;
        init_pair(1,info_f,info_b);
        wattron(txt, A_BOLD | COLOR_PAIR(1));
        c_info_off();

   }
    i = -1;
    prompy("[*] set warning color: "); GETSTRING(msg, 1);
    if(msg[0] != '\0' && isdigit(msg[0])) i = atoi(msg);
    if(i >= 0 && i <= 7) {

        warn_f = i;
        init_pair(4,warn_f,warn_b);
        wattron(txt, A_BOLD | COLOR_PAIR(4));
        c_warn_off();
    }
    i = -1;
    prompy("[*] set text color: "); GETSTRING(msg, 1);
    if(msg[0] != '\0' && isdigit(msg[0])) i = atoi(msg);
    if(i >= 0 && i <= 7) {

        text_f = i;
        init_pair(3,text_f,text_b);
        wattron(txt, A_NORMAL | COLOR_PAIR(3));
        c_text_off();
    }
    i = -1;
    prompy("[*] set bars foregroud color: "); GETSTRING(msg, 1);
    if(msg[0] != '\0' && isdigit(msg[0])) i = atoi(msg);
    if(i >= 0 && i <= 7) {

        bars_f = i;
        init_pair(2,bars_f,bars_b); color_set(2,NULL);
        c_prompt();
    }
    i = -1;
    prompy("[*] set bars backgroud color: "); GETSTRING(msg, 1);
    if(msg[0] != '\0' && isdigit(msg[0])) i = atoi(msg);
    if(i >= 0 && i <= 7) {

        bars_b = i;
        init_pair(2,bars_f,bars_b); color_set(2,NULL);
        c_prompt();
    }
}

void datez(void) {

    TIMESCAPE colorinfo(); wprintw(txt, "%s", ctime(&t));
}

int decoyz(void) {

    if(!con) { nocom(); return(0); }
    if(con && !decoy) {

        if(pthread_create(&dec,NULL,send_decoy,NULL) != 0) superbad();
        pthread_detach(pthread_self());
        colorinfo(); wprintw(txt, "decoy traffic: ON\n"); decoy = 1;
    }
    else if(con && decoy) {

        if(pthread_cancel(dec) != 0) superbad();
        colorinfo(); wprintw(txt, "decoy traffic: OFF\n"); decoy = 0;
    }
    return(0);
}

int disconnect(void) {

    colorinfo(); wprintw(txt, "disconnecting Tor, killing process\n");
    waitingfor();
    if(killtor()) return(1);
    nocom(); mutual(0); return(0);
}

int floodz(void) {

    if(!flood) {

        prompy("[*] set message rate per minute (10-50): "); GETSTRING(msg, 2);
        if(msg[0] == '\0' || isdigit(msg[0]) == 0 || isdigit(msg[1]) == 0)
            return(1);
        burst = (atoi(msg));
        if(burst < 10 || burst > 50) { burst = 0; return(1); }
        colorinfo();
        wprintw(txt, "antiflood: ON\n");
        tflo = time(&t); flood = 1;
    }
   else {

       colorinfo(); wprintw(txt, "antiflood: OFF\n");
       flood = 0; burst = 0; tflo = 0;
    }
    return(0);
}

int generate(void) {

    prompy("[*] generate new .onion? [y/n]: "); GETSTRING(msg, 1);
    if(msg[0] != 'y') return(1);
    else {

        waitingfor(); colorinfo(); wprintw(txt, "generating new .onion\n");
        saveonion();
        createtcdirs();
        fireuptor();
        char *b = hidden();
        if(b != NULL) { hm(); wprintw(txt, "current:  %s\n", b); free(b); }
    }
    return(0);
}

void infoz(void) {

    ibox = 1;
    char trin[FMT];
    char trout[FMT];
    memset(trin,0,sizeof(trin));
    memset(trout,0,sizeof(trout));
    strlcat(trin, ska(bytein), sizeof(trin));
    strlcat(trout, ska(byteout), sizeof(trout));
    INFOBOX
    i_info(); mvwprintw(info,1,23,"info"); i_info_off();
    mvwprintw(info,2,5,"real ip address\t\t%s", chk);
    mvwprintw(info,3,5,"tor exit node\t\t%s", chktor);
    mvwprintw(info,4,5,"tor proxy\t\t\t%s:%d", TPROXY, TPORT);
    mvwprintw(info,5,5,"tor process\t\t%d", tpid);
    if(undead == 0)
    mvwprintw(info,6,5,"tor keepalive is\t\toff");
    if(undead != 0) {

        elapse(tkep);
        mvwprintw(info,6,5,"tor keepalive in\t\t%d minutes (every %d)",
        (undead-min), undead);
    }
    mvwprintw(info,7,5,"tor version\t\t%s", torver);
    mvwprintw(info,8,5,"gnupg version\t\t%s", gpgver);
    mvwprintw(info,9,5,"tc daemon ip\t\t%s:%d", TCDAEMON, TCPORT);
    mvwprintw(info,10,5,"tc traffic in\t\t%s", trin);
    mvwprintw(info,11,5,"tc traffic out\t\t%s", trout);
    mvwprintw(info,12,5,"current windows is\t\t%d x %d", LINES, COLS);
    if(flood == 0)
    mvwprintw(info,13,5,"antiflood is\t\toff");
    if(flood != 0)

        mvwprintw(info,13,5,"antiflood burst at\t\t%d message/min", burst);
    if(!sym)

        mvwprintw(info,14,5,"encryption is\t\tasymmetric (pubkey)");
    if(sym && symg)

        mvwprintw(info,14,5,"encryption is\t\tsymmetric (gpg)");
    if(sym && symo)

        mvwprintw(info,14,5,"encryption is\t\tsymmetric (openssl)");

    wrefresh(info); prompy(""); noecho(); getch(); delwin(info); ibox = 0;
}

int join(void) {

    if(!con && sym) setsympass();
    if(!con) {

        prompy("[*] join to: "); GETSTRING(msg, N-1);
        if(msg[0] == '\0' || checkname(msg)) return(1);
        waitingfor();
        colorinfo(); wprintw(txt, "joining %s\n", msg);
        for(int i = 0; i < who; i++) {

            hm(); wprintw(txt, "%d. %s\n", i+1, o[i]);
        }
        con = 1;
        char buf[] = " has joined the conversation";
        sendinfos(buf);
    }
    else {

        colorwarn(); wprintw(txt, "already joined to %s\n", remo);
    }
    return(0);
}

int settorkeepalive(void) {

    if(!con) { nocom(); return(0); }
    if(!undead) {

        prompy("[*] how ofthen check tor (10-50) minute, 0 to disable: ");
        GETSTRING(msg, 2);
        if(msg[0] == '\0' || isdigit(msg[0]) == 0 || isdigit(msg[1]) == 0)
            return(1);
        undead = (atoi(msg));
        if(undead < 10 || undead > 50) { undead = 0; return(1); }
        colorinfo(); wprintw(txt, "Tor keepalive: ON\n"); tkep = time(&t);
    }
   else {

       colorinfo(); wprintw(txt, "Tor keepalive: OFF\n"); undead = 0; tkep = 0;
    }
    return(0);
}

void laziness(void) {

    colorinfo();
    if(!lazy) { wprintw(txt, "laziness: ON\n"); lazy = 1; }
    else { wprintw(txt, "laziness: OFF\n"); lazy = 0; }
}

void logging(void) {

    colorinfo();
    if(!logs) { wprintw(txt, "logging: ON\n"); logs = 1; }
    else { wprintw(txt, "logging: OFF\n"); logs = 0; }
}

int emote(void) {

    if(!con) { nocom(); return(0); }
    else {

        prompy("[*] notice: "); GETSTRING(msg, N*4);
        if(msg[0] == '\0') return(1);
        wprintw(txt, "%02d:%02d ", tm.tm_hour, tm.tm_min);
        c_whit(); wprintw(txt, "*notice* "); c_whit_off();
        c_warn(); wprintw(txt, "%s %s\n", nick, msg);
        c_warn_off();
        char buf[STR] = {"\0"};
        strlcat(buf, "<notice> ", sizeof(buf));
        strlcat(buf, nick, sizeof(buf));
        strlcat(buf, " ", sizeof(buf));
        strlcat(buf, msg, sizeof(buf));
        strlcpy(msg, buf, sizeof(msg));
        forward();
        addscrollback(msg);
    }
    return(0);
}

int newnick(void) {

    prompy("[*] new nick: "); GETSTRING(msg, N-1);
    if(msg[0] == '\0' || !strcmp(msg, nick)) return(1);
    char tmp[STR] = {"\0"};
    strlcpy(tmp, nick, sizeof(tmp));
    memset(nick, 0, sizeof(nick));
    strlcpy(nick, msg, sizeof(nick));
    invalidnick(nick);
    if(con) {

        char buf[STR] = {"\0"};
        strlcat(buf, tmp, sizeof(buf));
        strlcat(buf, " has changed nick to ", sizeof(buf));
        strlcat(buf, nick, sizeof(buf));
        strlcpy(msg, buf, sizeof(msg));
        forward();
    }
    return(0);
}

int part(void) {

    if(!con) { nocom(); return(0); }
    else {

        prompy("[*] set leave message: "); GETSTRING(way, N*3);
        if(way[0] == '\0') {

            char buf[] = " has left the conversation";
            sendinfos(buf); nocom(); mutual(0);
        }
        else {

            char buf[STR] = " has left the conversation: ";
            strlcat(buf, way, sizeof(buf));
            sendinfos(buf); nocom(); mutual(0);
        }
    }
    return(0);
}

void quit(void) {

    if(!con) cleanup(0);
    prompy("[*] set quit message: "); GETSTRING(way, N*3);
    if(way[0] == '\0') {

        char buf[] = " has quit the conversation";
        sendinfos(buf);
        if(pthread_cancel(srv) != 0) superbad();
        if(decoy) if(pthread_cancel(dec) != 0) superbad();
        cleanup(0);
    }
    else {

        char buf[STR] = " has quit the conversation: ";
        strlcat(buf, way, sizeof(buf));
        sendinfos(buf);
        if(pthread_cancel(srv) != 0) superbad();
        if(decoy) if(pthread_cancel(dec) != 0) superbad();
        cleanup(0);
    }
}

int dorestore(void) {

    hm(); c_info(); wprintw(txt, "<*> restore:\n"); c_info_off();
    char buf[STR] = {"\0"};
    strlcat(buf, "ls ", sizeof(buf));
    strlcat(buf, home, sizeof(buf));
    strlcat(buf, "/.tc/ | grep onion | cut -c 5-66", sizeof(buf));
    FILE *p = NULL;
    if((p = popen(buf, "r")) == NULL) return(1);
    int rest = 0;
    while(1) {

        fgets(restore[rest].onion, 64, p);
        if(restore[rest].onion[0] == '\0') break;
        restore[rest].onion[strcspn(restore[rest].onion,"\n")]='\0';
        rest++;
        if(rest >= N) break;
    }
    BADPCLOSE(p)
    for(int i = 0; i < rest; i++) {

        hm(); wprintw(txt, "  %d.[  ", i); c_whit();
        wprintw(txt, "%-63s", restore[i].onion); c_whit_off();
        wprintw(txt, " ]\n");
    }
    if(!rest) {

        colorwarn(); wprintw(txt, "no hidden services to restore\n");
        return(1);
    }
    hm(); wprintw(txt, "(total count: %d)\n", rest); draw();
    prompy("[*] choose .onion to restore: "); GETSTRING(msg, 1);
    if(msg[0] == '\0' || !isdigit(msg[0])) return(1);
    int oa = (atoi(msg));
    if(oa >= 0 && oa < rest) {

        waitingfor();
        colorinfo(); wprintw(txt, "restoring .onion\n");
        saveonion();
        buf[0] = '\0';
        if(!access(tchost, F_OK)) {

            colorwarn(); wprintw(txt, "%s still in place\n", tchost);
            return(1);
        }
        strlcat(buf, "mv ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/tor.", sizeof(buf));
        strlcat(buf, restore[oa].onion, sizeof(buf));
        strlcat(buf, " ", sizeof(buf));
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, "/.tc/tor", sizeof(buf));
        if(system(buf) == -1) {

            colorwarn(); wprintw(txt, "mv: %s\n", strerror(errno));
        }
        fireuptor();
        char *b = hidden();
        if(b != NULL) { hm(); wprintw(txt, "current:  %s\n", b); free(b); }
        if(con) nocom();
    }
    return(0);
}

int send_file(void) { /* sendfile() previos declared in socket.h */

    if(!con) { nocom(); return(0); }
    else {

        prompy("[*] file: "); GETSTRING(msg, MSG);
        if(msg[0] == '\0' || checkfile()) return(1);
        snd = 1;
        FILE *f = fopen(msg, "r");
        if(f == NULL) return(1);
        if(checkfsize(f, msg, 104857600, 0)) return(1); /* over 100Mb */
        if(grp) {

            char tmp[STR] = {"\0"};
            strlcpy(tmp, msg, sizeof(tmp));
            memset(msg, 0, sizeof(msg));
            char buf[] = " sending a file to group";
            sendinfos(buf);
            memset(msg, 0, sizeof(msg));
            strlcpy(msg, tmp, sizeof(msg));
        }
        waitingfor();
        (grp) ? gcrypt(NULL, NULL, msg) : ucrypt(NULL, NULL, msg);
        snd = 0;
    }
    return(0);
}

int incipit(char const *file) { /* XXX drawremote() is displayed anyway */

    FILE *f = fopen(file, "r"); if(f == NULL) return(1);
    char *b = NULL;
    size_t len = 0;
    int i = 0;
    pager = 1;
    int pos = row-3, l = 0;
    while (getline(&b, &len, f) != -1) {

        char *lin = b;
        if(lin == NULL) break;
        if(l++ ==  pos) {

            int cx = getch();
            if(cx == 'q') break; /* see Dr. Thomas in "Vieni avanti cretino" */
            else {

                l = 1; for(int j = 1; j < pos; j++) clrtoeol();
            }
        }
        int le = strlen(lin);
        lin[COLS - 7] = '\0'; /* 7 is time+1 chars: "12:08 " */
        hm(); wprintw(txt, "%s", lin);
        if(le > (COLS - 7)) wprintw(txt, "\n");
        i++;
    }
    free(b); fclose(f); pager = 0; return(0);
}

int showfile(char const *file) {

    if(file == NULL) {

        hm(); c_info(); wprintw(txt, "<*> bot's output:\n"); c_info_off();
        incipit(tcbot);
    }
    else {

        prompy("[*] file: "); GETSTRING(msg, MSG);
        if(msg[0] == '\0' || checkfile()) return(1);
        hm(); c_info(); wprintw(txt, "<*> %s:\n", msg); c_info_off();
        FILE *f = fopen(msg, "r"); if(f == NULL) return(1);
        unsigned char data[STR] = {"\0"};
        fread(data, 100, 1, f);
        if(fseek(f, 0L, SEEK_SET) != 0) errx(1, "reposition error");
        fclose(f);
        if(isbin(data, 100)) {

            colorwarn(); wprintw(txt, "%s seems a binary\n", msg);
            hexdumpo(data);
        }
        else incipit(msg);
    }
    return(0);
}

int strictness(void) {

    prompy("[*] authorization (c)lient (s)erver (m)utual: "); GETSTRING(msg, 1);
    if(msg[0] == 'c' || msg[0] == 's' || msg[0] == 'm') {

        waitingfor();
        loadauthkeys();
        switch((int)msg[0]) {

            case 'c': (nauth >= 1) ? selectk(0) : nokey(); break;
            case 's': (nprv >= 1) ? selectk(1) : nokey(); break;
            case 'm': if(nauth == 0 && nprv == 0) nokey();
                      (!strict) ? mutual(1) : mutual(0); break;
            default: break;
        }
        if(nauth >= 1 || nprv >= 1) {

            prompy("[*] restart Tor? [y/n]: "); GETSTRING(msg, 1);
            if(msg[0] != 'y') return(1);
            else {

                if(killtor()) return(1);
                if(!con) nocom();
                fireuptor();
                if(strict) {

                    colorinfo();
                    wprintw(txt,"onion authorization: ON\n");
                }
                else {
                    colorinfo(); wprintw(txt,"onion authorization: OFF\n");
                }
            }
        }
    }
    return(0);
}

int symmetric(void) {

    if(!con && !sym) { nocom(); return(0); }
    if(!con && sym) {

        colorinfo(); wprintw(txt, "symmetric encryption: OFF\n");
        sym = 0;  symg = 0; symo = 0; nocom();
    }
    else if(con && !sym) { setsympass(); sym = 1; }
    else if(con && sym) {

        char buf[] = " has switched OFF symmetric encyption";
        sendinfos(buf);
        colorinfo(); wprintw(txt, "symmetric encryption: OFF\n");
        sym = 0; symg = 0; symo = 0;
    }
    return(0);
}

int topic(char *m) {

    if(m == NULL) {

        prompy("[*] new topic: "); GETSTRING(msg, N*3);
        if(msg[0] == '\0') return(1);
        strlcpy(tag, msg, (N*3)+1);

        if(con) {

            char buf[STR] = {"\0"};
            strlcat(buf, nick, sizeof(buf));
            strlcat(buf, " changed the topic to: ", sizeof(buf));
            strlcat(buf, msg, sizeof(buf));
            strlcpy(msg, buf, sizeof(msg));
            forward();
        }
    }
    else strlcpy(tag, m, (N*3)+1);
    return(0);
}

void uptime(void) {

    elapse(tstart);
    colorinfo();
    if(day > 1 || day == 0)
    wprintw(txt, "tc uptime: %u days, %02u:%02u:%02u\n", day, hour, min, sec);
    if(day == 1)
    wprintw(txt, "tc uptime: %u day, %02u:%02u:%02u\n", day, hour, min, sec);
}

void version(void) {

    colorinfo(); wprintw(txt, "tc version: %s, build %s\n", VERSION, BUILD);
}

int setvoice(void) {

    if(!con) { nocom(); return(0); }
    if(con && !voice) {

        colorinfo(); wprintw(txt, "text to speech: ON\n"); voice = 1;
    }
    else if(con && voice) {

        colorinfo(); wprintw(txt, "text to speech: OFF\n"); voice = 0;
    }
    return(0);
}

int whoz(void) {

    if(!con) { nocom(); return(0); }
    if(con) {

        hm(); c_info(); wprintw(txt, "<*> talking to:\n"); c_info_off();
        for(int i = 0; i < who; i++) {

            hm(); wprintw(txt, "%d. %s\n", i+1, o[i]);
        }
    }
    return(0);
}

int whois(void) {

    prompy("[*] contact: "); GETSTRING(msg, N-1);
    if(msg[0] == '\0') return(0);
    showcontacts(msg);
    return(0);
}

void chat(void) {

    tstart = time(&t);
    if((mainwin = initscr()) == NULL) exit(1);
    getmaxyx(mainwin, row, col);
    raw();
    txt = newwin(row-2, col, 1, 0);
    scrollok(txt, TRUE);
    pad = newpad(STR, (COLS-1));
    keypad(pad, TRUE);
    clear();
    mutual(0);
    start_color();
    draw();
    curs_set(0);
    keypad(mainwin, TRUE);
    struct sigaction act1, act2, act3;
    act1.sa_handler = &decrypt; act1.sa_flags = SA_RESTART;
    act2.sa_handler = &cleanup; act2.sa_flags = SA_RESTART;
    act3.sa_handler = &xtermresize; act3.sa_flags = SA_RESTART;
    sigaction(SIGUSR2, &act1, NULL);
    sigaction(SIGINT, &act2, NULL);
    sigaction(SIGWINCH, &act3, NULL);
    unlink(TCS); shred(TCT); unlink(TCR); shred(TCN);
    if(pthread_create(&clk, NULL, clocker, NULL) != 0) superbad();
    pthread_detach(pthread_self());
    if(pthread_create(&srv, NULL, listener, NULL) != 0) superbad();
    pthread_detach(pthread_self());
    invalidnick(nick);
    strlcpy(cipher, "AES256", sizeof(cipher));
    while (1) {

        noecho();
        deleteln();
        draw();
        redrawwin(txt);
        keyboard();
        if(msg[0] == '\0') continue;
        history(msg);
        in[0] = '\0';
        if(msg[0] == '/') {

            switch (hash(msg)) {

                case 0xad2  : algoz();             continue; /* /algo       */
                case 0x806  :                                /* /away       */
                case 0x612  : awayz();             continue; /* /a          */
                case 0x234e :                                /* /bot        */
                case 0x613  : botz();              continue; /* /b          */
                case 0x1d41 : callz();             continue; /* /call       */
                case 0x131b : camuflagez();        continue; /* /camuflage  */
                case 0x26ca :                                /* /circuit    */
                case 0x614  : torcircuit();        continue; /* /c          */
                case 0x2113 : clearz();            continue; /* /clear      */
                case 0x2051 : colorz();            continue; /* /colors     */
                case 0x105c : datez();             continue; /* /date       */
                case 0x2a72 :                                /* /decoy      */
                case 0x615  : decoyz();            continue; /* /d          */
                case 0x6da  : disconnect();        continue; /* /disconnect */
                case 0xc9   :                                /* /exit       */
                case 0x616  : cleanup(0);          continue; /* /e          */
                case 0x221e : flagz();             continue; /* /flags      */
                case 0xc7   :                                /* /flood      */
                case 0x617  : floodz();            continue; /* /f          */
                case 0x1efc : generate();          continue; /* /generate   */
                case 0xaf2  :                                /* /group      */
                case 0x618  : showgroups();        continue; /* /g          */
                case 0x1282 :                                /* /help       */
                case 0x619  : help();              continue; /* /h          */
                case 0x23e3 : history(NULL);       continue; /* /history    */
                case 0x25ba :                                /* /info       */
                case 0x61a  : infoz();             continue; /* /i          */
                case 0x1c01 :                                /* /join       */
                case 0x61b  : join();              continue; /* /j          */
                case 0x15fd :                                /* /keys       */
                case 0x61c  : printkeys();         continue; /* /k          */
                case 0x241f : settorkeepalive();   continue; /* /keepalive  */
                case 0x24af : laziness();          continue; /* /lazy       */
                case 0x1d64 :                                /* /log        */
                case 0x61d  : logging();           continue; /* /l          */
                case 0x106b :                                /* /me         */
                case 0x61e  : emote();             continue; /* /m          */
                case 0xb94  : showcontacts(NULL);  continue; /* /names      */
                case 0x2434 :                                /* /nick       */
                case 0x61f  : newnick();           continue; /* /n          */
                case 0x17c1 :                                /* /part       */
                case 0x621  : part();              continue; /* /p          */
                case 0x2881 :                                /* /quit       */
                case 0x622  : quit();              continue; /* /q          */
                case 0x1dae :                                /* /restore    */
                case 0x623  : dorestore();         continue; /* /r          */
                case 0x281e : send_file();         continue; /* /send       */
                case 0x82c  :                                /* /show       */
                case 0x624  : showfile(msg);       continue; /* /s          */
                case 0xad4  : strictness();        continue; /* /strict     */
                case 0x22c4 : symmetric();         continue; /* /symmetric  */
                case 0x21d3 :                                /* /topic      */
                case 0x625  : topic(0);            continue; /* /t          */
                case 0x143d :                                /* /uptime     */
                case 0x626  : uptime();            continue; /* /u          */
                case 0x2983 :                                /* /version    */
                case 0x165c : version();           continue; /* /ver        */
                case 0x110c :                                /* /voice      */
                case 0x627  : setvoice();          continue; /* /v          */
                case 0x1a77 :                                /* /who        */
                case 0x628  : whoz();              continue; /* /w          */
                case 0xc1b  : whois();             continue; /* /whois      */
            }
        }
        if(!con) continue;
        redrawwin(txt);
        c_text();
        hm(); wprintw(txt,"<%s> %s\n", nick, msg);
        c_text_off();
        draw();
        addscrollback(NULL);
        if(logs) logger(nick, msg);
        (grp) ? gcrypt(nick, msg, NULL) : ucrypt(nick, msg, NULL);
        memset(msg, 0, sizeof(msg));
    }
}

void localfilez(void) {

    if(strlen(home) > MSG) errx(1, "$HOME is too long");
    if(strlen(trm) > MSG) errx(1, "$TERM is too long");
    char v[250][250]; /* 250*250=62500 write jibberish if passed */
    char *path[NFILE] = {"/.tc/.tcs","/.tc/.tct","/.tc/.tcr","/.tc/.tcn",
    "/.tc/tc.conf","/.tc/tc.log","/.tc/.lock","/.tc/.bot","/.tc","/.tc/tc.rc",
    "/.tc/tor","/.tc/tor/hidden_service/hostname","/.tc/tor/auth",
    "/.tc/tor/hidden_service/authorized_clients","/.tc/tc_XXXXXXXXXXXXXX",
    "/.tc/tc.ver","/.tc/tor/hidden_service","/.tc/tc.tgz","/.tc/bot.sh",
    "/.tc/pem"};
    unsigned long l = 0;
    for(int i = 0; i < NFILE; i++) {

        char buf[STR] = {"\0"};
        strlcat(buf, home, sizeof(buf));
        strlcat(buf, path[i], sizeof(buf));
        strlcpy(v[i], buf, sizeof(v[i]));
        l = l + sizeof(buf);
        assert(l < (sizeof(v) - 1));
        if(l > (sizeof(v) - 1000)) errx(1, "path overflow detected: %lu",l);
    }
    TCS = v[0]; TCT = v[1]; TCR = v[2]; TCN = v[3]; tcconf = v[4]; tclog = v[5];
    tclock = v[6]; tcbot = v[7]; tcdir = v[8]; tcrc = v[9]; tctordir = v[10];
    tchost = v[11]; tcauthdir = v[12]; tcauthclidir = v[13]; tcrcvfile = v[14];
    tcver = v[15]; tchiddendir = v[16]; tctgz = v[17]; tcbotsh = v[18];
    pem = v[19];
}

int main(int argc, char *argv[]) {

    fprintf(stderr, "%s\n", tag);
    if(argc < 2) usage();
    if((home = getenv("HOME")) == 0) exit(1);
    if((trm = getenv("TERM")) == 0) exit(1);

    #ifdef openbsd
    unveil("/dev/null", "w");              unveil("/etc/resolv.conf", "r");
    unveil("/bin", "r");                   unveil("/usr/share/terminfo", "r");
    unveil("/bin/ls", "x");                unveil("/bin/mv", "x");
    unveil("/bin/ps", "x");                unveil("/bin/sh", "x");
    unveil("/bin/tar", "x");               unveil("/usr/bin/cut", "x");
    unveil("/usr/bin/awk", "x");           unveil("/usr/bin/grep", "x");
    unveil("/usr/bin/head", "x");          unveil("/usr/bin/pkill", "x");
    unveil("/usr/sbin/openssl", "x");      unveil("/usr/local/bin/tor", "x");
    unveil("/usr/local/bin/gpg", "x");     unveil("/usr/local/bin/espeak", "x");

    if(pledge("stdio rpath wpath cpath proc exec tty inet unveil dns",NULL)==-1)
        err(1, "pledge");
    #endif

    localfilez();

    #ifdef openbsd /* after localfilez() need path vars loaded */
    jail = (const char *)tcdir;
    unveil(jail, "rwxc");
    unveil(NULL, NULL);
    #endif

    createtcdirs();
    FILE *f = NULL;
    f = fopen(tcrc, "r"); (f == NULL) ? createrc() : (void)fclose(f);
    f = fopen(tcconf, "r"); (f == NULL) ? createtcconf() : (void)fclose(f);

    f = fopen(tclock, "r");
    if(f != NULL) {

        char p[10] = {"\0"};
        fgets(p, 10, f);
        int run = kill(atoi(p), 0);
        if(run == 0) errx(1, "already running (pid: %s)", p);
        else {

            fclose(f);
            unlink(tclock);
            f = fopen(tclock, "w"); if(f == NULL) errx(1,"tclock");
            fprintf(f, "%d", getpid());
            fclose(f);
        }
    }
    else {

        f = fopen(tclock, "w"); if(f == NULL) errx(1,"tclock");
        fprintf(f, "%d", getpid());
        fclose(f);
    }

    f = fopen(tcconf, "r");
    if(f == NULL) errx(1, "can't open tc.conf");
    if(!checkfsize(f, tcconf, MAX_FSIZE, 1)) loadconfig();
    gettorver(); getgnupgver(); loadkeysid();
    int i;
    while((i = getopt(argc, argv, "n:s:")) != -1) {

        switch(i) {

            case 'n': strlcpy(nick, optarg, N); chat(); exit(0);
            case 's': strlcpy(nick, optarg, N); sym = 1; chat(); exit(0);
            default:  usage();
        }
    }
    usage();
    return(0);
}
